/* -------------------------------------------------------------------------- 
 * Copyright (c) 2018-2019 Kneron Inc. All rights reserved.
 *
 *      Name:    mpu.c
 *      Purpose: Kneron NCPU memory configuration and protection
 *
 *---------------------------------------------------------------------------*/

#include <string.h>
#include "kneron_mozart.h"
#include "io.h"
#include "base.h"

void kdrv_mpu_config(void);

#define MPU_DEFAULT_MAP 0x04
#define MPU_XN 1U << 28
#define MPU_64KB (ARM_MPU_REGION_SIZE_64KB << 1)
#define MPU_128KB (ARM_MPU_REGION_SIZE_128KB << 1)
#define MPU_96KB_SRD (0xC0 << 8) // disable sub region 6 and 7
#define MPU_256MB (ARM_MPU_REGION_SIZE_256MB << 1)
#define MPU_64MB (ARM_MPU_REGION_SIZE_64MB << 1)
#define MPU_4GB (ARM_MPU_REGION_SIZE_4GB << 1)
#define MPU_2pGB_SRD (0xE0 << 8)  // exclude sub-region 5, 6 and 7 (0xA000-0xFFFF uses default)
#define MPU_RW (ARM_MPU_AP_FULL << 24)
#define MPU_RO (ARM_MPU_AP_RO << 24)
#define MPU_NA (ARM_MPU_AP_NONE << 24)
#define MPU_CACHE  (0x0F << 16) // TEX = 001, S = 1, C = 1, B = 1, -> full cache WBWA shared
#define MPU_DEV_S  (0x05 << 16) // TEX = 000, S = 1, C = 0, B = 1, -> shared device
#define MPU_EN 1
void kdrv_mpu_config(void);

void kdrv_mpu_config(void)
{
    ARM_MPU_SetRegionEx(0, 0x00000000, MPU_NA | MPU_DEV_S | MPU_4GB | MPU_2pGB_SRD | MPU_XN | MPU_EN);
    ARM_MPU_SetRegionEx(1, 0x00000000, MPU_RO | MPU_CACHE | MPU_64KB | MPU_EN);
    ARM_MPU_SetRegionEx(2, 0x0FFF0000, MPU_RW | MPU_CACHE | MPU_64KB | MPU_XN | MPU_EN);
    ARM_MPU_SetRegionEx(3, 0x20200000, MPU_RW | MPU_CACHE | MPU_128KB | MPU_96KB_SRD | MPU_XN | MPU_EN);
    ARM_MPU_SetRegionEx(4, 0x30000000, MPU_RW | MPU_CACHE | MPU_256MB | MPU_XN | MPU_EN);
    ARM_MPU_SetRegionEx(5, 0x60000000, MPU_RW | MPU_CACHE | MPU_64MB | MPU_XN | MPU_EN);
    ARM_MPU_Enable(MPU_DEFAULT_MAP);  // enable the background mem map
//    readw(0x64005678);  // testing for memory fault only
}
