/*
 * Kneron Power Manager driver
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
 
#include "cmsis_os2.h"
#include "cmsis_armcc.h"
#include "kneron_mozart.h"
#include "scu_ipc.h"
#include "power_mgr.h"
#include "dbg.h"

extern struct ncpu_to_scpu_s *out_comm_p;
#define MMFAR 0xE000ED34

static void power_mgr_exception(void)
{
    /* report error status */
    out_comm_p->status = STATUS_ERR;
    /* notify scpu */
    scu_ipc_trigger_to_scpu_int();
    /* wait for scpu to reset */
    power_mgr_cpu_idle();
}

register unsigned int _msp __asm("msp");
register unsigned int _psp __asm("psp");
register unsigned int _lr  __asm("lr");
static unsigned int stack, pc;

static void ncpu_hard_fault(void)
{
    if (_lr & 4) {
        stack = _psp;
        pc = stack + 24;
    }
    else {
        stack = _msp;
        pc = stack + 32;
    }

    err_msg("ncpu: hard fault @ %08X, PC = %08X, LR = %08X, SP = %08X\n", *(u32*)MMFAR,
        *(u32*)pc, *(u32*)(pc-4), (u32)pc+8);
    power_mgr_exception();
}

static void ncpu_mem_mnmt(void)
{
    if (_lr & 4) {
        stack = _psp;
        pc = stack + 24;
    }
    else {
        stack = _msp;
        pc = stack + 32;
    }

    err_msg("ncpu: memory fault @ 0x%08X, PC = %08X, LR = %08X, SP = %08X\n", *(u32*)MMFAR,
        *(u32*)pc, *(u32*)(pc-4), (u32)pc+8);
    power_mgr_exception();
}

static void ncpu_bus_fault(void)
{
    err_msg("ncpu: ncpu_bus_fault !\n");
    power_mgr_exception();
}

static void ncpu_usage_fault(void)
{
    err_msg("scpu: ncpu_usage_fault !\n");
    power_mgr_exception();
}


void power_mgr_error_notify(uint32_t code, void *object_id)
{
    err_msg("ncpu: exception: code=%d, object_id=0x%p\n", code, object_id);
    power_mgr_exception();
}

__NO_RETURN void power_mgr_cpu_idle(void)
{
    while(1) {
        __WFI();
    }
}

void power_mgr_init(void)
{
    NVIC_SetVector(HardFault_IRQn, (uint32_t)ncpu_hard_fault);
    NVIC_SetVector(MemoryManagement_IRQn, (uint32_t)ncpu_mem_mnmt);
    NVIC_SetVector(BusFault_IRQn, (uint32_t)ncpu_bus_fault);
    NVIC_SetVector(UsageFault_IRQn, (uint32_t)ncpu_usage_fault);
}

