#ifndef USE_KDP_CAMERA
#ifdef V2K_ENABLE
#include "config/board_kdp520.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <cmsis_os2.h>
#include "kneron_mozart.h"
#include "base.h"
#include "utility.h"
#include "io.h"
#include "kdrv_scu_ext.h"
#include "kdrv_clock.h"
#include "framework/init.h"
#include "framework/ioport.h"
#include "framework/kdp_list.h"
#include "framework/framework_errno.h"
#include "framework/mm.h"
#include "framework/spinlock.h"
#include "framework/event.h"
#include "framework/framework_driver.h"
#include "framework/v2k.h"
#include "media/v2k_subdev.h"
#include "media/camera/sys_camera.h"
//#include "kdrv_lcdc.h"
//#include "media/display/display.h"
#include "kmdw_memxfer.h"
#include "kdrv_ddr.h"
#include "kmdw_console.h"
#include "kdp_drv_fb_mgr.h"

//#define SOURCE_FROM_PATTERN    

#define CSI2RX_REG_VIDR         0x00
#define CSI2RX_REG_DIDR         0x01
#define CSI2RX_REG_CR           0x04 //Control Register
#define CSI2RX_REG_VSCR         0x05 //DPI V Sync Control Register
#define CSI2RX_REG_ECR          0x06
#define CSI2RX_REG_TCNR         0x08
#define CSI2RX_REG_HRTVR        0x0A //HS RX Timeout Value Register
#define CSI2RX_REG_FIUR         0x0B 
#define CSI2RX_REG_ITR          0x12 //Initialization Timer Register
#define CSI2RX_REG_VSTR0        0x14 //DPI VC0 V Sync Timing Register
#define CSI2RX_REG_HSTR0        0x15 //DPI VC0 H Sync Timing Register
#define CSI2RX_REG_VSTR1        0x16
#define CSI2RX_REG_HSTR1        0x17
#define CSI2RX_REG_VSTR2        0x18
#define CSI2RX_REG_HSTR2        0x19
#define CSI2RX_REG_VSTR3        0x1A
#define CSI2RX_REG_HSTR3        0x1B
#define CSI2RX_REG_MCR          0x1C //DPI Mapping Control Register
#define CSI2RX_REG_VSTER        0x1E
#define CSI2RX_REG_HPNR         0x20 //DPI Horizontal Pixel Number
#define CSI2RX_REG_PECR         0x28 //PPI Enable Control Register
#define CSI2RX_REG_DLMR         0x2A
#define CSI2RX_REG_CSIERR       0x30
#define CSI2RX_REG_INTSTS       0x33
#define CSI2RX_REG_ESR          0x34
#define CSI2RX_REG_DPISR        0x38
#define CSI2RX_REG_INTER        0x3C
#define CSI2RX_REG_FFR          0x3D
#define CSI2RX_REG_DPCMR        0x48
#define CSI2RX_REG_FRR          0x4C
#define CSI2RX_REG_PFTR         0x50
#define CSI2RX_REG_PUDTR        0x58
#define CSI2RX_REG_FRCR         0x80
#define CSI2RX_REG_FNR          0x88
#define CSI2RX_REG_BPGLR        0x90
#define CSI2RX_REG_BPGHR        0x91

#define MAX_VIDEO_MEM           8 << 20 

#define CSIRX_0_BASE            CSIRX_FTCSIRX100_PA_BASE
#define CSIRX_1_BASE            CSIRX_FTCSIRX100_1_PA_BASE

/* DPI V Sync Control Register (VSCR, Address = 0x05) */
#define CSIRX_REG_VSCR_SET_VSTU0(addr, val) SET_MASKED_BIT(addr, val, 0)

/* PPI Enable Control Register (PECR, Address = 0x28) */
#define CSIRX_REG_PECR_GET_PEC(addr)        GET_BITS(addr, 0, 7)
#define CSIRX_REG_PECR_SET_PEC(addr, val)   SET_MASKED_BITS(addr, val, 0, 7)

/* Data Lane Mapping Register 0 */
#define CSIRX_REG_0_DLMR0                   (CSIRX_0_BASE + 0x2A)
#define CSIRX_REG_0_DLMR0_GET_L1()          GET_BITS(CSIRX_REG_0_DLMR0, 4, 6)
#define CSIRX_REG_0_DLMR0_GET_L0()          GET_BITS(CSIRX_REG_0_DLMR0, 0, 2)
#define CSIRX_REG_1_DLMR0                   (CSIRX_1_BASE + 0x2A)
#define CSIRX_REG_1_DLMR0_GET_L1()          GET_BITS(CSIRX_REG_1_DLMR0, 4, 6)
#define CSIRX_REG_1_DLMR0_GET_L0()          GET_BITS(CSIRX_REG_1_DLMR0, 0, 2)


#define CSIRX_REG_0_FEATURE0                (CSIRX_0_BASE + 0x40)
#define CSIRX_REG_1_FEATURE0                (CSIRX_1_BASE + 0x40)
/* DPI FIFO Address Depth */
#define CSIRX_REG_0_FEATURE0_GET_DFAD()     GET_BITS(CSIRX_REG_0_FEATURE0, 5, 7)
#define CSIRX_REG_1_FEATURE0_GET_DFAD()     GET_BITS(CSIRX_REG_1_FEATURE0, 5, 7)
/* Lane Number */
#define CSIRX_REG_0_FEATURE0_GET_LN()       GET_BITS(CSIRX_REG_0_FEATURE0, 2, 4)
#define CSIRX_REG_1_FEATURE0_GET_LN()       GET_BITS(CSIRX_REG_1_FEATURE0, 2, 4)
/* Virtual channel number */
#define CSIRX_REG_0_FEATURE0_GET_VCN()      GET_BITS(CSIRX_REG_0_FEATURE0, 0, 1)
#define CSIRX_REG_1_FEATURE0_GET_VCN()      GET_BITS(CSIRX_REG_1_FEATURE0, 0, 1)

#define CSIRX_REG_0_FEATURE6                (CSIRX_0_BASE + 0x46)
#define CSIRX_REG_1_FEATURE6                (CSIRX_1_BASE + 0x46)
/* ASC Indicator */
#define CSIRX_REG_0_FEATURE6_GET_ASC()      GET_BIT(CSIRX_REG_0_FEATURE6, 7)
#define CSIRX_REG_1_FEATURE6_GET_ASC()      GET_BIT(CSIRX_REG_1_FEATURE6, 7)
/* APB Indicator */
#define CSIRX_REG_0_FEATURE6_GET_APB()      GET_BIT(CSIRX_REG_0_FEATURE6, 6)
#define CSIRX_REG_1_FEATURE6_GET_APB()      GET_BIT(CSIRX_REG_1_FEATURE6, 6)
/* I2C Indicator */
#define CSIRX_REG_0_FEATURE6_GET_I2C()      GET_BIT(CSIRX_REG_0_FEATURE6, 5)
#define CSIRX_REG_1_FEATURE6_GET_I2C()      GET_BIT(CSIRX_REG_1_FEATURE6, 5)
/* DPCM_EN Indicator */
#define CSIRX_REG_0_FEATURE6_GET_DPCM()     GET_BIT(CSIRX_REG_0_FEATURE6, 4)
#define CSIRX_REG_1_FEATURE6_GET_DPCM()     GET_BIT(CSIRX_REG_1_FEATURE6, 4)
/* CHK_DATA Indicator */
#define CSIRX_REG_0_FEATURE6_GET_CD()       GET_BIT(CSIRX_REG_0_FEATURE6, 3)
#define CSIRX_REG_1_FEATURE6_GET_CD()       GET_BIT(CSIRX_REG_1_FEATURE6, 3)
/* DUAL_PIXEL Indicator */
#define CSIRX_REG_0_FEATURE6_GET_DP()       GET_BIT(CSIRX_REG_0_FEATURE6, 2)
#define CSIRX_REG_1_FEATURE6_GET_DP()       GET_BIT(CSIRX_REG_1_FEATURE6, 2)
/* FRC Indicator */
#define CSIRX_REG_0_FEATURE6_GET_FRC()      GET_BIT(CSIRX_REG_0_FEATURE6, 1)
#define CSIRX_REG_1_FEATURE6_GET_FRC()      GET_BIT(CSIRX_REG_1_FEATURE6, 1)
/* DPI_PG Indicator */
#define CSIRX_REG_0_FEATURE6_GET_PG()       GET_BIT(CSIRX_REG_0_FEATURE6, 0)
#define CSIRX_REG_1_FEATURE6_GET_PG()       GET_BIT(CSIRX_REG_1_FEATURE6, 0)

#define CSIRX_REG_0_FEATURE7                (CSIRX_0_BASE + 0x47)
#define CSIRX_REG_1_FEATURE7                (CSIRX_1_BASE + 0x47)
/* OCTA_PIXEL Indicator */
#define CSIRX_REG_0_FEATURE7_GET_OP()       GET_BIT(CSIRX_REG_0_FEATURE7, 4)
#define CSIRX_REG_1_FEATURE7_GET_OP()       GET_BIT(CSIRX_REG_1_FEATURE7, 4)
/* MONO_RGB_EN Indicator */
#define CSIRX_REG_0_FEATURE7_GET_MR()       GET_BIT(CSIRX_REG_0_FEATURE7, 3)
#define CSIRX_REG_1_FEATURE7_GET_MR()       GET_BIT(CSIRX_REG_1_FEATURE7, 3)
/* CRC_CHK Indicator */
#define CSIRX_REG_0_FEATURE7_GET_CRC()      GET_BIT(CSIRX_REG_0_FEATURE7, 2)
#define CSIRX_REG_1_FEATURE7_GET_CRC()      GET_BIT(CSIRX_REG_1_FEATURE7, 2)
/* LANE_SWAP Indicator */
#define CSIRX_REG_0_FEATURE7_GET_SWAP()     GET_BIT(CSIRX_REG_0_FEATURE7, 1)
#define CSIRX_REG_1_FEATURE7_GET_SWAP()     GET_BIT(CSIRX_REG_1_FEATURE7, 1)
/* QUAD_PIXEL Indicator */
#define CSIRX_REG_0_FEATURE7_GET_QP()       GET_BIT(CSIRX_REG_0_FEATURE7, 0)
#define CSIRX_REG_1_FEATURE7_GET_QP()       GET_BIT(CSIRX_REG_1_FEATURE7, 0)


#define CSIRX_REG_0_BPGLR0                  (CSIRX_0_BASE + 0x90)
#define CSIRX_REG_1_BPGLR0                  (CSIRX_1_BASE + 0x90)
#define CSIRX_REG_0_BPGLR0_SET_VLN(val)     SET_MASKED_BITS(CSIRX_REG_0_BPGLR0, val, 0, 7)
#define CSIRX_REG_1_BPGLR0_SET_VLN(val)     SET_MASKED_BITS(CSIRX_REG_1_BPGLR0, val, 0, 7)

#define CSIRX_REG_0_BPGHR0                  (CSIRX_0_BASE + 0x91)
#define CSIRX_REG_1_BPGHR0                  (CSIRX_1_BASE + 0x91)
#define CSIRX_REG_0_BPGHR0_SET_PT(val)      SET_MASKED_BITS(CSIRX_REG_0_BPGHR0, val, 6, 7)
#define CSIRX_REG_1_BPGHR0_SET_PT(val)      SET_MASKED_BITS(CSIRX_REG_1_BPGHR0, val, 6, 7)
#define CSIRX_REG_0_BPGHR0_SET_PS(val)      SET_MASKED_BIT(CSIRX_REG_0_BPGHR0, val, 5)
#define CSIRX_REG_1_BPGHR0_SET_PS(val)      SET_MASKED_BIT(CSIRX_REG_1_BPGHR0, val, 5)
#define CSIRX_REG_0_BPGHR0_SET_GE(val)      SET_MASKED_BIT(CSIRX_REG_0_BPGHR0, val, 4)
#define CSIRX_REG_1_BPGHR0_SET_GE(val)      SET_MASKED_BIT(CSIRX_REG_1_BPGHR0, val, 4)
#define CSIRX_REG_0_BPGHR0_SET_VLN(val)     SET_MASKED_BITS(CSIRX_REG_0_BPGHR0, val, 0, 3)
#define CSIRX_REG_1_BPGHR0_SET_VLN(val)     SET_MASKED_BITS(CSIRX_REG_1_BPGHR0, val, 0, 3)

//#define DEBUG_CSI2RX_FEATURE
//#define DEBUG_CSI2RX_REG
//#define CSI2RX_PATTERN_GEN
#define D2A_REG_CTRL            0x00
#define D2A_REG_FNC             0x04
#define D2A_REG_P0ADDR          0x08
#define D2A_REG_P1ADDR          0x0C
#define D2A_REG_ICT             0x10
#define D2A_REG_IS              0x14
#define D2A_REG_ST              0x18
#define D2A_REG_PT              0x1C
#define D2A_REG_FIU             0x20

/* D2A Control Register */

/* D2A interrupt control register */
#define D2A_ICT_TILE_AVG_D_CE   0x20
#define D2A_ICT_WRD_CE          0x01

/* D2A interrupt status register */
#define D2A_IS_AVG_D            0x20
#define D2A_IS_WRD              0x01

/* D2A status register */
#define D2A_ST_PG               0x3

/* D2A Packet type register */
#define D2A_PT_YUV422           0x1E
#define D2A_PT_RGB565           0x22
#define D2A_PT_RAW10            0x2B
#define D2A_PT_RAW8             0x2A


#define FLAG_CSI2RX_START       0x0001
#define FLAG_CSI2RX_ISR       0x0010

//#define TILE_AVG_CALC_ENABLE

#define TILE_AVG_SIZE_128       0x00000000
#define TILE_AVG_SIZE_64        BIT16
#define TILE_AVG_SIZE_32        BIT17
#define TILE_AVG_SIZE_VAL       TILE_AVG_SIZE_128


#define WILL_UPDATE_AT_NEXT_VERSION

static osThreadId_t m_tid_csi2rx_array[2] = { 0 };
static int m_page_num_array[2] = { 2, 2 };
static int m_csi2rx_driver_inst_num = 0;

extern const struct s_kdp_memxfer kdp_memxfer_module;
extern struct kdrv_clock_node clock_node_csirx0_hs_csi;
extern struct kdrv_clock_node clock_node_csirx0_hs_vc0;
extern struct kdrv_clock_node clock_node_csirx0_lp;

extern struct kdp520_csi2rx_context_lite *g_csi2rx0_ctx;
extern struct kdp520_csi2rx_context_lite *g_csi2rx1_ctx;

extern void small_delay(int us);

enum mipi_csi2rx_id {
    MIPI_CSI2RX_0 = 0,
    MIPI_CSI2RX_1
};

struct kdp520_fb {
    struct video_buf vb;	
    struct kdp_list list;
};

struct kdp520_csi2rx_context {
    u32 ai_mem_addr;
    u32 ai_mem_addr_tmp;
    u32 vw_mem_addr;
    u32 ov_mem_addr;
    u32 dpi2ahb_base;
    u32 base;
    u32 sid;
    enum mipi_csi2rx_id id;
    enum v2k_field field;
    struct sys_camera_host cam_h;
    void *alloc_ctx;
    struct kdp520_fb *active;
    struct kdp_list fb_list;
    struct spinlock_t lock; /* Protects video buffer lists */
    int page_num_done;
};

static inline struct sys_camera_device *sys_camera_from_vb2q(
        const struct video_buf_queue *vq) {
    return container_of(vq, struct sys_camera_device, vb_q);
}

static struct kdp520_fb *to_kdp520_vb(struct video_buf *vb)
{
    return container_of(vb, struct kdp520_fb, vb);
}

extern int lcdc_frame_addr;

static void camera_isr(int cam_idx, struct kdp520_csi2rx_context_lite *ctx)
{   
    u32 sta_is, sta_st;
#ifdef USE_FRAME_BUFFER_DRIVER
    u32 buf_addr[2];
    static int buf0_idx[2] = {-1,-1};
    static int buf1_idx[2] = {-1,-1};
#endif

    sta_is = inw(ctx->dpi2ahb_base + D2A_REG_IS);

#ifdef TILE_AVG_CALC_ENABLE
    if(cam_idx == 1) {
        if(sta_is & D2A_IS_AVG_D) { //tile average ready
            u32 total = 0;
            u8 t8;
            for(int i = 0; i < 5; i++) {
                u32 tmp = inw(ctx->dpi2ahb_base + 0x28 + i * 10);
                t8 = tmp & 0xff;
                total += t8;
                t8 = (tmp & 0xff00) >> 8;
                total += t8;
                t8 = (tmp & 0xff0000) >> 16;
                total += t8;
                t8 = (tmp & 0xff000000) >> 24;
                total += t8;
            }
            t8 = total / 20;
        }
        outw(ctx->dpi2ahb_base + D2A_REG_FNC, 0x2);
    }
#endif

    if (sta_is & D2A_IS_WRD) {
        sta_st = inw(ctx->dpi2ahb_base + D2A_REG_ST);
        if (m_page_num_array[cam_idx] != sta_st) // 1: page 1 done 2: page 0 done
        {
#ifdef USE_FRAME_BUFFER_DRIVER
            if (sta_st == 2) {
                if (buf0_idx[cam_idx] != -1)
                    kdp_fb_mgr_write_done(cam_idx, buf0_idx[cam_idx]);
                buf_addr[cam_idx] =  kdp_fb_mgr_next_write(cam_idx, &buf0_idx[cam_idx]);
                outw(ctx->dpi2ahb_base + D2A_REG_P0ADDR, buf_addr[cam_idx]);
                //osThreadFlagsSet(m_tid_csi2rx_array[cam_idx], FLAG_CSI2RX_ISR);
            } else {
                if (buf1_idx[cam_idx] != -1)
                    kdp_fb_mgr_write_done(cam_idx, buf1_idx[cam_idx]);
                buf_addr[cam_idx] =  kdp_fb_mgr_next_write(cam_idx, &buf1_idx[cam_idx]);
                outw(ctx->dpi2ahb_base + D2A_REG_P1ADDR, buf_addr[cam_idx]);
            }
#else
            if (sta_st == 2) {
                outw(ctx->dpi2ahb_base + D2A_REG_P0ADDR, ctx->ai_mem_addr_tmp);
                if (cam_idx == 0)
                    lcdc_frame_addr = ctx->vw_mem_addr;
                osThreadFlagsSet(m_tid_csi2rx_array[cam_idx], FLAG_CSI2RX_ISR);
            }
#endif
            m_page_num_array[cam_idx] = sta_st;
        }
    }   

    outw(ctx->dpi2ahb_base + D2A_REG_IS, sta_is);
}

void dpi2ahb_0_isr(void)
{   
    struct kdp520_csi2rx_context_lite *ctx = g_csi2rx0_ctx;

    camera_isr(0, ctx);
}

void dpi2ahb_1_isr(void)
{
    struct kdp520_csi2rx_context_lite *ctx = g_csi2rx1_ctx;

    camera_isr(1, ctx);
}

static void camera_thread(int cam_idx, struct kdp520_csi2rx_context *ctx, struct kdp520_csi2rx_context_lite *ctx_lite)
{
    u32 flags;
    struct kdp520_fb *fb;

    for (;;) {
        flags = osThreadFlagsWait(FLAG_CSI2RX_ISR, osFlagsWaitAny, osWaitForever);
        osThreadFlagsClear(flags);
        fb = ctx->active;
        if (fb) {
            //user push a buffer into working queue
            kdp_list_del_init(&fb->list);
            if (!kdp_list_empty(&ctx->fb_list)) {
                ctx->active = (struct kdp520_fb*)
                &container_of(ctx->fb_list.next, struct kdp520_fb, list)->vb;
            } else
                ctx->active = NULL;
            if (cam_idx == 0)
                videobuf_ready_to_dequeue(FLAGS_VB_DONE_0, &fb->vb, VB_STATE_DONE);
            else
                videobuf_ready_to_dequeue(FLAGS_VB_DONE_1, &fb->vb, VB_STATE_DONE);
            outw(ctx->dpi2ahb_base + D2A_REG_P0ADDR, ctx_lite->ai_mem_addr);
        }
        else
            osDelay(20);
    }
}

static void csi2rx_thread_0(void *argument)
{
    struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context*)argument;

    camera_thread(0, ctx, g_csi2rx0_ctx);
}

static void csi2rx_thread_1(void *argument)
{
    struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context*)argument;

    camera_thread(1, ctx, g_csi2rx1_ctx);
}

void dpi2ahb_enable(struct kdp520_csi2rx_context *csi2rx_ctx, 
        struct v2k_format *format)
{
    struct kdp520_csi2rx_context *ctx = csi2rx_ctx;

    u32 val = 0;
    DSG("[%s] ctx->dpi2ahb_base=0x%x", __func__, ctx->dpi2ahb_base);
    switch (ctx->sid) {
    case SENSOR_ID_HMX2056:
        val |= 0x3000;
        break;
    case SENSOR_ID_OV9286:
        val |= 0x3008;
        break;
    case SENSOR_ID_HMXRICA:
        val |= 0x3008;
        break;
    case SENSOR_ID_GC2145:
        DSG("[%s] SENSOR_ID_GC2145", __func__);
        val |= 0x2800;
        break;
    case SENSOR_ID_SC132GS:
        DSG("[%s] SENSOR_ID_SC132GS", __func__);
        val |= 0x3000;
        break;
    }

#ifdef TILE_AVG_CALC_ENABLE
    if(ctx->sid == SENSOR_ID_SC132GS) { //for nir
        val |= TILE_AVG_SIZE_VAL;
        outw(ctx->dpi2ahb_base + D2A_REG_FNC, 0x2);
        outw(ctx->dpi2ahb_base + D2A_REG_CTRL, val);
        outw(ctx->dpi2ahb_base + D2A_REG_ICT, D2A_ICT_WRD_CE | D2A_ICT_TILE_AVG_D_CE);//0x3f);
    } else {
        outw(ctx->dpi2ahb_base + D2A_REG_FNC, 0x0);
        outw(ctx->dpi2ahb_base + D2A_REG_CTRL, val);
        outw(ctx->dpi2ahb_base + D2A_REG_ICT, D2A_ICT_WRD_CE);//0x3f);
    }
#else
    outw(ctx->dpi2ahb_base + D2A_REG_FNC, 0x0);
    outw(ctx->dpi2ahb_base + D2A_REG_CTRL, val);
    outw(ctx->dpi2ahb_base + D2A_REG_ICT, D2A_ICT_WRD_CE);//0x3f);
#endif

    outw(ctx->dpi2ahb_base + D2A_REG_P0ADDR, ctx->ai_mem_addr);
    outw(ctx->dpi2ahb_base + D2A_REG_P1ADDR, ctx->vw_mem_addr);

#ifdef SOURCE_FROM_PATTERN
    if (MIPI_CSI2RX_0 == ctx->id) {
    #if IMGSRC_0_FORMAT == IMAGE_FORMAT_YCBCR
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_YUV422);
    #elif IMGSRC_0_FORMAT == IMAGE_FORMAT_RGB565
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_RGB565);
    #elif IMGSRC_0_FORMAT == IMAGE_FORMAT_RAW10
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_RAW10);
    #elif IMGSRC_0_FORMAT == IMAGE_FORMAT_RAW8
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_RAW8);
    #endif
    }
    else if (MIPI_CSI2RX_1 == ctx->id) {
    #if IMGSRC_1_FORMAT == IMAGE_FORMAT_YCBCR
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_YUV422);
    #elif IMGSRC_1_FORMAT == IMAGE_FORMAT_RGB565
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_RGB565);
    #elif IMGSRC_1_FORMAT == IMAGE_FORMAT_RAW10
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_RAW10);
    #elif IMGSRC_1_FORMAT == IMAGE_FORMAT_RAW8
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_RAW8);
    #endif
    }
#else
    switch (format->pixelformat) {
    case V2K_PIX_FMT_YCBCR:
        DSG("[%s] V2K_PIX_FMT_YCBCR ", __func__);
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_YUV422);
        break;        
    case V2K_PIX_FMT_RGB565:
        DSG("[%s] V2K_PIX_FMT_RGB565 ", __func__);
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_RGB565);
        break;
    case V2K_PIX_FMT_RAW10:	
        DSG("[%s] V2K_PIX_FMT_RAW10 ", __func__);
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_RAW10);
        break;
    case V2K_PIX_FMT_RAW8:
        DSG("[%s] V2K_PIX_FMT_RAW8 ", __func__);
        outw(ctx->dpi2ahb_base + D2A_REG_PT, D2A_PT_RAW8);
        break;
    default:;
    }
#endif

    outw(ctx->dpi2ahb_base + D2A_REG_FIU, 0x1);

    if (MIPI_CSI2RX_0 == ctx->id) {
        DSG("[%s] D2A_FTDPI2AHB_IRQ", __func__);
        NVIC_SetVector((IRQn_Type)D2A_FTDPI2AHB_IRQ, (u32)dpi2ahb_0_isr);
        NVIC_EnableIRQ(D2A_FTDPI2AHB_IRQ);
    }
    else if (MIPI_CSI2RX_1 == ctx->id) {
        DSG("[%s] D2A_FTDPI2AHB_1_IRQ", __func__);
        NVIC_SetVector((IRQn_Type)D2A_FTDPI2AHB_1_IRQ, (u32)dpi2ahb_1_isr);
        NVIC_EnableIRQ(D2A_FTDPI2AHB_1_IRQ);
    }
}

void csi2rx_init(struct kdp520_csi2rx_context *csi2rx_ctx, 
        struct v2k_format *format)
{ 
    int val;
    struct kdp520_csi2rx_context *ctx = csi2rx_ctx;

    if (MIPI_CSI2RX_0 == ctx->id) {
        DSG(" csi2rx(0) : csi2rx_init ctx->base=0x%x", ctx->base);
    }
    else if (MIPI_CSI2RX_1 == ctx->id) {
        DSG(" csi2rx(1) : csi2rx_init ");
    }
    
    u16 width = ROUND_UP(format->width, 4);
    DSG("width=%d", width);
    // val = inw(ctx->base + CSI2RX_REG_HPNR);
    // val = (val&(~0xffff)) | width;
    outw(ctx->base + CSI2RX_REG_HPNR , width);

    val = inw(ctx->base + CSI2RX_REG_CR);
    //val = (val&(~0xffff)) | 0x010d; 
    val = (val&(~0xff)) | 0x0d; 
    outw(ctx->base + CSI2RX_REG_CR , val);
    
    val = inw(ctx->base + CSI2RX_REG_MCR);
    val = (val&(~0xff)) | 0x00;//0x22;
    outw(ctx->base + CSI2RX_REG_MCR , val);

#ifdef SOURCE_FROM_PATTERN    
    CSIRX_REG_PECR_SET_PEC(ctx->base + CSI2RX_REG_PECR, 1);
    
    //outw(ctx->base + CSI2RX_REG_BPGLR, 0x01); 
    val = (inw(ctx->base + CSI2RX_REG_BPGLR) & 0xFF);
    DSG(" CSI2RX_REG_BPGLR val=%x", val);
    
    val = (inw(ctx->base + CSI2RX_REG_BPGHR) & 0x0F);
        //val = 0;

    if (MIPI_CSI2RX_0 == ctx->id) {
    #if IMGSRC_0_FORMAT == IMAGE_FORMAT_RGB565
        val |= 0x40;         //RAW10
    #elif IMGSRC_0_FORMAT == IMAGE_FORMAT_RAW10
        val |= 0x40;         //RAW10
    #elif IMGSRC_0_FORMAT == IMAGE_FORMAT_RAW8
        val |= 0x00;         //RAW8
    #endif
    }
    else if (MIPI_CSI2RX_1 == ctx->id) {
    #if IMGSRC_1_FORMAT == IMAGE_FORMAT_RGB565
        val |= 0x40;         //RAW10
    #elif IMGSRC_1_FORMAT == IMAGE_FORMAT_RAW10
        val |= 0x40;         //RAW10
    #elif IMGSRC_1_FORMAT == IMAGE_FORMAT_RAW8
        val |= 0x00;         //RAW8
    #endif        
    }
    //val |= 0x20; //swap          
    outw(ctx->base + CSI2RX_REG_BPGHR, val); 
    val = inw(ctx->base + CSI2RX_REG_BPGHR);
    //DSG(" CSI2RX_REG_BPGHR val=%x", val);
#endif    

    switch (ctx->sid) {
    case SENSOR_ID_OV9286:
    case SENSOR_ID_HMXRICA:
    case SENSOR_ID_SC132GS:
        //val = inw(ctx->base + CSI2RX_REG_PFTR);
        val = 160;//100; // * 4 = 400 pixels
        outw(ctx->base + CSI2RX_REG_PFTR , val);
        break;
    case SENSOR_ID_GC2145:
        CSIRX_REG_VSCR_SET_VSTU0(ctx->base, 1);
        break;
    default:;
    }
    
    
}

#define PLL3_INITED_IN_SYTEM_INIT

static int kdp520_csi2rx_clock_start(struct sys_camera_host *cam_h)
{
#if 0
    struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context *)cam_h->priv;
#endif

    return 0;
#if 0
    if (MIPI_CSI2RX_0 == ctx->id) {

        DSG("%s MIPI_CSI2RX_0", __func__);

        SCU_EXTREG_CLK_EN1_SET_csirx0_TxEscClk(1);
        SCU_EXTREG_CLK_EN1_SET_csirx0_vc0(1);
        SCU_EXTREG_CLK_EN1_SET_csirx0_csi(1);        
        //0717
        //SCU_EXTREG_CLK_EN1_SET_csirx1_TxEscClk(1);
        //SCU_EXTREG_CLK_EN1_SET_csirx1_vc0(1);
        //SCU_EXTREG_CLK_EN1_SET_csirx1_csi(1);    
        
    #ifndef PLL3_INITED_IN_SYTEM_INIT        
        SCU_EXTREG_CLK_EN1_SET_csirx0_TxEscClk(0);
        SCU_EXTREG_CLK_EN1_SET_csirx0_vc0(0);
        SCU_EXTREG_CLK_EN1_SET_csirx0_csi(0);
        
        {
            SCU_EXTREG_CLK_EN1_SET_csirx1_TxEscClk(0);
            SCU_EXTREG_CLK_EN1_SET_csirx1_vc0(0);
            SCU_EXTREG_CLK_EN1_SET_csirx1_csi(0);
        }

        // Disable PLL output (clock gated)
        SCU_EXTREG_CLK_EN0_SET_pll3_out2(0);
        SCU_EXTREG_CLK_EN0_SET_pll3_out1(0);

        // Power down PLL.
        SCU_EXTREG_PLL3_SETTING_SET_en(0);
        
        //Set the new M/N/P values.
        kdrv_clock_mgr_change_pll3_clock(2, 0x10c, 2);

        // For PLL510 series, wait for 40us reset time
        small_delay(40);    

        // Enable PLL.
        SCU_EXTREG_PLL3_SETTING_SET_en(1);

        // Wait for PLL locking time. (PLL110: 50us, PLL510: 350us)
        small_delay(350);

        // Enable PLL output.
        SCU_EXTREG_CLK_EN0_SET_pll3_out2(1);
        SCU_EXTREG_CLK_EN0_SET_pll3_out1(1);

        SCU_EXTREG_CLK_EN1_SET_csirx0_TxEscClk(1);
        SCU_EXTREG_CLK_EN1_SET_csirx0_vc0(1);
        SCU_EXTREG_CLK_EN1_SET_csirx0_csi(1);

        {
            SCU_EXTREG_CLK_EN1_SET_csirx1_TxEscClk(1);
            SCU_EXTREG_CLK_EN1_SET_csirx1_vc0(1);
            SCU_EXTREG_CLK_EN1_SET_csirx1_csi(1);
        }
    #endif
    #ifdef DEBUG_CSI2RX_FEATURE
        DSG("mipi 0 feature DPI FIFO Address Depth=%u", CSIRX_REG_0_FEATURE0_GET_DFAD());
        DSG("mipi 0 feature Lane Number=%u", CSIRX_REG_0_FEATURE0_GET_LN());
        DSG("mipi 0 feature Virtual channel number=%u", CSIRX_REG_0_FEATURE0_GET_VCN());
        DSG("mipi 0 feature ASC=%u", CSIRX_REG_0_FEATURE6_GET_ASC());
        DSG("mipi 0 feature APB=%u", CSIRX_REG_0_FEATURE6_GET_APB());
        DSG("mipi 0 feature I2C=%u", CSIRX_REG_0_FEATURE6_GET_I2C());
        DSG("mipi 0 feature DPCM_EN=%u", CSIRX_REG_0_FEATURE6_GET_DPCM());
        DSG("mipi 0 feature CHK_DATA=%u", CSIRX_REG_0_FEATURE6_GET_CD());
        DSG("mipi 0 feature DUAL_PIXEL=%u", CSIRX_REG_0_FEATURE6_GET_DP());
        DSG("mipi 0 feature FRC=%u", CSIRX_REG_0_FEATURE6_GET_FRC());
        DSG("mipi 0 feature DPI_PG=%u", CSIRX_REG_0_FEATURE6_GET_PG());    

        DSG("mipi 0 feature OCTA_PIXEL=%u", CSIRX_REG_0_FEATURE7_GET_OP());
        DSG("mipi 0 feature MONO_RGB_EN=%u", CSIRX_REG_0_FEATURE7_GET_MR());
        DSG("mipi 0 feature CRC_CHK=%u", CSIRX_REG_0_FEATURE7_GET_CRC());

        val = CSIRX_REG_0_FEATURE7_GET_SWAP();
        DSG("mipi 0 feature LANE_SWAP=%u", val);    
        if (val == 0) {
            DSG("mipi 0 physical data lane == logical data lane");
        } else {
            DSG("mipi 0 physical data lane != logical data lane");
        }
        DSG("mipi 0 feature QUAD_PIXEL=%u", CSIRX_REG_0_FEATURE7_GET_QP());

        DSG("mipi 0 feature L1=%u", CSIRX_REG_0_DLMR0_GET_L1());
        DSG("mipi 0 feature L0=%u", CSIRX_REG_0_DLMR0_GET_L0());
    #endif
        
    }
    else if (MIPI_CSI2RX_1 == ctx->id) {
        
        DSG("%s MIPI_CSI2RX_1", __func__);
            
        SCU_EXTREG_CLK_EN1_SET_csirx0_TxEscClk(1);
        SCU_EXTREG_CLK_EN1_SET_csirx0_vc0(1);
        SCU_EXTREG_CLK_EN1_SET_csirx0_csi(1);
//james
        SCU_EXTREG_CLK_EN1_SET_csirx1_TxEscClk(1);
        SCU_EXTREG_CLK_EN1_SET_csirx1_vc0(1);
        SCU_EXTREG_CLK_EN1_SET_csirx1_csi(1);         

//    #ifndef PLL3_INITED_IN_SYTEM_INIT
//        SCU_EXTREG_CLK_EN1_SET_csirx1_TxEscClk(0);
//        SCU_EXTREG_CLK_EN1_SET_csirx1_vc0(0);
//        SCU_EXTREG_CLK_EN1_SET_csirx1_csi(0);
//        {
//            SCU_EXTREG_CLK_EN0_SET_pll3_out2(0);
//            SCU_EXTREG_CLK_EN0_SET_pll3_out1(0);
//        }
//        //SCU_EXTREG_CLK_MUX_SEL_SET_csirx1_clk(1);
//        small_delay(40);small_delay(40);    

//        // Wait for PLL locking time. (PLL110: 50us, PLL510: 350us)
//        small_delay(350);small_delay(350);
//        //divider
//        SCU_EXTREG_CLK_DIV7_SET_csirx1_TxEscClk_pll3(5);
//        SCU_EXTREG_CLK_DIV7_SET_csi1_csi_pll3(15);
//        SCU_EXTREG_CLK_DIV7_SET_csi1_vc0_pll3(4);
//        {
//            SCU_EXTREG_CLK_EN0_SET_pll3_out2(1);
//            SCU_EXTREG_CLK_EN0_SET_pll3_out1(1);        
//        }        
//        SCU_EXTREG_CLK_EN1_SET_csirx0_TxEscClk(1);
//        SCU_EXTREG_CLK_EN1_SET_csirx0_vc0(1);
//        SCU_EXTREG_CLK_EN1_SET_csirx0_csi(1);

//        SCU_EXTREG_CLK_EN1_SET_csirx0_TxEscClk(1);
//        SCU_EXTREG_CLK_EN1_SET_csirx0_vc0(1);
//        SCU_EXTREG_CLK_EN1_SET_csirx0_csi(1);

//        SCU_EXTREG_CLK_EN1_SET_csirx1_TxEscClk(1);
//        SCU_EXTREG_CLK_EN1_SET_csirx1_vc0(1);
//        SCU_EXTREG_CLK_EN1_SET_csirx1_csi(1);
//    #endif
    #ifdef DEBUG_CSI2RX_FEATURE
        DSG("mipi 1 feature DPI FIFO Address Depth=%u", CSIRX_REG_1_FEATURE0_GET_DFAD());
        DSG("mipi 1 feature Lane Number=%u", CSIRX_REG_1_FEATURE0_GET_LN());
        DSG("mipi 1 feature Virtual channel number=%u", CSIRX_REG_1_FEATURE0_GET_VCN());
        DSG("mipi 1 feature ASC=%u", CSIRX_REG_1_FEATURE6_GET_ASC());
        DSG("mipi 1 feature APB=%u", CSIRX_REG_1_FEATURE6_GET_APB());
        DSG("mipi 1 feature I2C=%u", CSIRX_REG_1_FEATURE6_GET_I2C());
        DSG("mipi 1 feature DPCM_EN=%u", CSIRX_REG_1_FEATURE6_GET_DPCM());
        DSG("mipi 1 feature CHK_DATA=%u", CSIRX_REG_1_FEATURE6_GET_CD());
        DSG("mipi 1 feature DUAL_PIXEL=%u", CSIRX_REG_1_FEATURE6_GET_DP());
        DSG("mipi 1 feature FRC=%u", CSIRX_REG_1_FEATURE6_GET_FRC());
        DSG("mipi 1 feature DPI_PG=%u", CSIRX_REG_1_FEATURE6_GET_PG());    

        DSG("mipi 1 feature OCTA_PIXEL=%u", CSIRX_REG_1_FEATURE7_GET_OP());
        DSG("mipi 1 feature MONO_RGB_EN=%u", CSIRX_REG_1_FEATURE7_GET_MR());
        DSG("mipi 1 feature CRC_CHK=%u", CSIRX_REG_1_FEATURE7_GET_CRC());

        val = CSIRX_REG_1_FEATURE7_GET_SWAP();
        DSG("mipi 1 feature LANE_SWAP=%u", val);    
        if (val == 0) {
            DSG("mipi 1 physical data lane == logical data lane");
        } else {
            DSG("mipi 1 physical data lane != logical data lane");
        }
        DSG("mipi 1 feature QUAD_PIXEL=%u", CSIRX_REG_1_FEATURE7_GET_QP());

        DSG("mipi 1 feature L1=%u", CSIRX_REG_1_DLMR0_GET_L1());
        DSG("mipi 1 feature L0=%u", CSIRX_REG_1_DLMR0_GET_L0());
    #endif
    }    

    return 0;
#endif
}

static void kdp520_csi2rx_clock_stop(struct sys_camera_host *cam_h)
{
    struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context *)cam_h->priv;
    
    DSG("%s", __func__);
    
    if (MIPI_CSI2RX_0 == ctx->id) {
        masked_outw(SCU_EXTREG_CSIRX_CTRL0, 
                    0,
                   (SCU_EXTREG_CSIRX_CTRL0_ClkLnEn | SCU_EXTREG_CSIRX_CTRL0_Enable));
    }
    else if (MIPI_CSI2RX_1 == ctx->id) { 
        masked_outw(SCU_EXTREG_CSIRX_CTRL1, 
                    0,
                   (SCU_EXTREG_CSIRX_CTRL1_ClkLnEn | SCU_EXTREG_CSIRX_CTRL1_Enable));
    }
}

static int kdp520_csi2rx_reset(struct sys_camera_host *cam_h)//struct core_device *core_d)
{
    //struct sys_camera_host *sch = to_sys_camera_host(&core_d->pin_ctx);
    struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context*)cam_h->priv;

    if (MIPI_CSI2RX_0 == ctx->id) {
    
        DSG("%s MIPI_CSI2RX_0", __func__);
        
    #if 1

        masked_outw( SCU_EXTREG_CSIRX_CTRL0, 
                    ((SCU_EXTREG_CSIRX_CTRL0_ClkLnEn | SCU_EXTREG_CSIRX_CTRL0_Enable) |
                     (SCU_EXTREG_CSIRX_CTRL0_apb_rst_n | SCU_EXTREG_CSIRX_CTRL0_pwr_rst_n | 
                      SCU_EXTREG_CSIRX_CTRL0_sys_rst_n)),
                    ((SCU_EXTREG_CSIRX_CTRL0_ClkLnEn | SCU_EXTREG_CSIRX_CTRL0_Enable) |
                     (SCU_EXTREG_CSIRX_CTRL0_apb_rst_n | SCU_EXTREG_CSIRX_CTRL0_pwr_rst_n | 
                      SCU_EXTREG_CSIRX_CTRL0_sys_rst_n)));
        
        masked_outw( SCU_EXTREG_CSIRX_CTRL1, 
                    ((SCU_EXTREG_CSIRX_CTRL1_ClkLnEn | SCU_EXTREG_CSIRX_CTRL1_Enable) |
                     (SCU_EXTREG_CSIRX_CTRL1_apb_rst_n | SCU_EXTREG_CSIRX_CTRL1_pwr_rst_n | 
                      SCU_EXTREG_CSIRX_CTRL1_sys_rst_n)),
                    ((SCU_EXTREG_CSIRX_CTRL1_ClkLnEn | SCU_EXTREG_CSIRX_CTRL1_Enable) |
                     (SCU_EXTREG_CSIRX_CTRL1_apb_rst_n | SCU_EXTREG_CSIRX_CTRL1_pwr_rst_n | 
                      SCU_EXTREG_CSIRX_CTRL1_sys_rst_n)));
        
        masked_outw( SCU_EXTREG_DPI2AHB_CTRL, 
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n |
                     SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n),
                    
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n |
                     SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n));   

        masked_outw( SCU_EXTREG_DPI2AHB_CTRL, 
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n_1 |
                     SCU_EXTREG_DPI2AHB_CTRL_sys_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n_1),
                    
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n_1 |
                     SCU_EXTREG_DPI2AHB_CTRL_sys_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n_1));
                     
    #if V2K_ENABLE_TYPE == V2K_ENABLE_GC2145 || V2K_ENABLE_TYPE == V2K_ENABLE_GC2145_SC132GS
        outb(MIPIRX_PHY_CSR_PA_BASE + 0x11, 0x7);
    #else
        outb(MIPIRX_PHY_CSR_PA_BASE + 0x11, 0x3);
    #endif
    
    #else
        
        masked_outw( SCU_EXTREG_CSIRX_CTRL0, 
                    (SCU_EXTREG_CSIRX_CTRL0_apb_rst_n | SCU_EXTREG_CSIRX_CTRL0_pwr_rst_n | 
                     SCU_EXTREG_CSIRX_CTRL0_sys_rst_n),
                    (SCU_EXTREG_CSIRX_CTRL0_apb_rst_n | SCU_EXTREG_CSIRX_CTRL0_pwr_rst_n |
                     SCU_EXTREG_CSIRX_CTRL0_sys_rst_n));  

        //dpi2ahb_0 reset
        masked_outw( SCU_EXTREG_DPI2AHB_CTRL, 
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n |
                     SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n),
                    
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n |
                     SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n));
    #endif
    

    } 
    else if (MIPI_CSI2RX_1 == ctx->id) {
        /* mipi csi2rx 1 */      
        DSG("%s MIPI_CSI2RX_1", __func__);
        
    #if 1
        masked_outw( SCU_EXTREG_CSIRX_CTRL0, 
                    ((SCU_EXTREG_CSIRX_CTRL0_ClkLnEn | SCU_EXTREG_CSIRX_CTRL0_Enable) |
                     (SCU_EXTREG_CSIRX_CTRL0_apb_rst_n | SCU_EXTREG_CSIRX_CTRL0_pwr_rst_n | 
                      SCU_EXTREG_CSIRX_CTRL0_sys_rst_n)),
                    ((SCU_EXTREG_CSIRX_CTRL0_ClkLnEn | SCU_EXTREG_CSIRX_CTRL0_Enable) |
                     (SCU_EXTREG_CSIRX_CTRL0_apb_rst_n | SCU_EXTREG_CSIRX_CTRL0_pwr_rst_n | 
                      SCU_EXTREG_CSIRX_CTRL0_sys_rst_n)));
        
        masked_outw( SCU_EXTREG_CSIRX_CTRL1, 
                    ((SCU_EXTREG_CSIRX_CTRL1_ClkLnEn | SCU_EXTREG_CSIRX_CTRL1_Enable) |
                     (SCU_EXTREG_CSIRX_CTRL1_apb_rst_n | SCU_EXTREG_CSIRX_CTRL1_pwr_rst_n | 
                      SCU_EXTREG_CSIRX_CTRL1_sys_rst_n)),
                    ((SCU_EXTREG_CSIRX_CTRL1_ClkLnEn | SCU_EXTREG_CSIRX_CTRL1_Enable) |
                     (SCU_EXTREG_CSIRX_CTRL1_apb_rst_n | SCU_EXTREG_CSIRX_CTRL1_pwr_rst_n | 
                      SCU_EXTREG_CSIRX_CTRL1_sys_rst_n)));
        
        masked_outw( SCU_EXTREG_DPI2AHB_CTRL, 
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n |
                     SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n),
                    
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n |
                     SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n));   

        masked_outw( SCU_EXTREG_DPI2AHB_CTRL, 
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n_1 |
                     SCU_EXTREG_DPI2AHB_CTRL_sys_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n_1),
                    
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n_1 |
                     SCU_EXTREG_DPI2AHB_CTRL_sys_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n_1));
                     
	#if V2K_ENABLE_TYPE == V2K_ENABLE_SC132GS_GC2145
        outb(MIPIRX_PHY_CSR_1_PA_BASE + 0x11, 0x7);                     
    #else
        outb(MIPIRX_PHY_CSR_1_PA_BASE + 0x11, 0x3);
    #endif     
   
        masked_outw( SCU_EXTREG_CSIRX_CTRL0, 
                    (SCU_EXTREG_CSIRX_CTRL0_apb_rst_n | SCU_EXTREG_CSIRX_CTRL0_pwr_rst_n | 
                     SCU_EXTREG_CSIRX_CTRL0_sys_rst_n),
                    (SCU_EXTREG_CSIRX_CTRL0_apb_rst_n | SCU_EXTREG_CSIRX_CTRL0_pwr_rst_n |
                     SCU_EXTREG_CSIRX_CTRL0_sys_rst_n));  
        
        masked_outw( SCU_EXTREG_CSIRX_CTRL1, 
                    (SCU_EXTREG_CSIRX_CTRL1_apb_rst_n | SCU_EXTREG_CSIRX_CTRL1_pwr_rst_n | 
                     SCU_EXTREG_CSIRX_CTRL1_sys_rst_n ),
                    (SCU_EXTREG_CSIRX_CTRL1_apb_rst_n | SCU_EXTREG_CSIRX_CTRL1_pwr_rst_n |
                     SCU_EXTREG_CSIRX_CTRL1_sys_rst_n ));        

        //dpi2ahb_1 reset
        masked_outw( SCU_EXTREG_DPI2AHB_CTRL, 
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n_1 |
                     SCU_EXTREG_DPI2AHB_CTRL_sys_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n_1),
                    
                    (SCU_EXTREG_DPI2AHB_CTRL_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_apb_rst_n_1 |
                     SCU_EXTREG_DPI2AHB_CTRL_sys_rst_n_1 | SCU_EXTREG_DPI2AHB_CTRL_pwr_rst_n_1));  

    #endif
    
    #if V2K_ENABLE_TYPE == V2K_ENABLE_GC2145
        outb(MIPIRX_PHY_CSR_1_PA_BASE + 0x11, 0x7);
    #else
        outb(MIPIRX_PHY_CSR_1_PA_BASE + 0x11, 0x3);
    #endif
    }        

    return 0;
}

static int kdp520_csi2rx_power(struct core_device *core_d, u8 on)
{
    struct sys_camera_host *cam_h = to_sys_camera_host(&core_d->pin_ctx);
    struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context*)cam_h->priv;
    
    //DSG("%s", __func__);
    
    u32 val = 0;    
    
    if (MIPI_CSI2RX_0 == ctx->id) {
        if (on)
            val = (SCU_EXTREG_CSIRX_CTRL0_ClkLnEn | SCU_EXTREG_CSIRX_CTRL0_Enable);

        masked_outw(SCU_EXTREG_CSIRX_CTRL0, 
                    val,
                   (SCU_EXTREG_CSIRX_CTRL0_ClkLnEn | SCU_EXTREG_CSIRX_CTRL0_Enable));
    }
    else if (MIPI_CSI2RX_1 == ctx->id) {
        if (on)
            val = (SCU_EXTREG_CSIRX_CTRL1_ClkLnEn | SCU_EXTREG_CSIRX_CTRL1_Enable);

        masked_outw(SCU_EXTREG_CSIRX_CTRL1, 
                    val,
                   (SCU_EXTREG_CSIRX_CTRL1_ClkLnEn | SCU_EXTREG_CSIRX_CTRL1_Enable));
    }    

    return 0;
}

static int kdp520_csi2rx_querycap(struct sys_camera_host *cam_h,
        struct v2k_capability *cap)
{
    strcpy(cap->desc, "Kdp520 camera");
    cap->capabilities = V2K_CAP_VIDEO_CAPTURE | V2K_CAP_STREAMING | V2K_CAP_DEVICE_CAPS;

    return 0;
}

static int kdp520_csi2rx_set_fmt(struct sys_camera_device *cam_d,
        struct v2k_format *format)
{
    int ret;
    struct sys_camera_host *cam_h = to_sys_camera_host(cam_d->parent_pin);
    //struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context *)cam_h->priv;
    struct v2k_subdev *sd = sys_camera_to_subdev(cam_d);
    const struct sys_camera_format_xlate *xlate;

    //DSG("<%s:%s> width=%d height=%d", __FILE__, __func__, format->width, format->height);

    xlate = sys_camera_xlate_by_fourcc(cam_d, format->pixelformat); //fourcc
    //DSG("%s sys_camera_xlate_by_fourcc xlate=%x", __func__, xlate);    
    if (!xlate) {
        //  DSG("<%s> Format %x not found", __func__, want_fmt->pixelformat);
        return -KDP_FRAMEWORK_ERRNO_INVALA;
    }

    dpi2ahb_enable((struct kdp520_csi2rx_context*)cam_h->priv, format);
    csi2rx_init((struct kdp520_csi2rx_context*)cam_h->priv, format);

    //DSG("[%s] v2k_subdev_call start", __func__);
    ret = v2k_subdev_call(sd, set_fmt, format);
    //DSG("[%s] v2k_subdev_call ret=%x", __func__, ret);

    // in the next driver version, we will add the video renderer to one of subdevs
#ifdef WILL_UPDATE_AT_NEXT_VERSION
#endif

    cam_d->current_fmt = xlate;

    return ret;
}

static int kdp520_csi2rx_set_bus_param(struct sys_camera_device *cam_d)
{
    //DSG("[%s]", __func__);
    //set hsync / vsync / pclk / 
    return 0;
}

static int kdp520_videobuf_setup(struct video_buf_queue *vq,
        unsigned int *num_buffers, 
        unsigned int *num_planes,
        unsigned int *size, void **alloc_ctx)
{
    struct sys_camera_device *cam_d = sys_camera_from_vb2q(vq);
    struct sys_camera_host *cam_h = to_sys_camera_host(cam_d->parent_pin);
    struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context *)cam_h->priv;

    if (*num_buffers == 0) return -KDP_FRAMEWORK_ERRNO_INVALA;

    *num_planes = 1;//2;
    *size = cam_d->sizeimage;
    *alloc_ctx = ctx->alloc_ctx;

    if (*size * *num_buffers * *num_planes > MAX_VIDEO_MEM) 
        *num_buffers = (MAX_VIDEO_MEM) / (*size * *num_planes);

    if (*num_buffers > 2)
        *num_buffers = 2;

    return 0;
}

static int kdp520_videobuf_init(struct video_buf *vb)
{
    struct video_buf_queue *vq = vb->vb_queue;
    struct sys_camera_device *cam_d = sys_camera_from_vb2q(vq);
    struct sys_camera_host *cam_h = to_sys_camera_host(cam_d->parent_pin);
    struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context *)cam_h->priv;
    struct kdp520_fb *fb = to_kdp520_vb(vb);

    DSG("[%s] ctx->id=%d ctx->dpi2ahb_base=%x mem_priv=%x", 
        __func__, ctx->id, ctx->dpi2ahb_base, (u32)vb->plane.mem_priv);    

//    {
//        switch (vb->index) {
//        case 0:
//            outw(ctx->dpi2ahb_base + D2A_REG_P0ADDR, (unsigned long)vb->plane.mem_priv);
//            break;
//        case 1:
//            outw(ctx->dpi2ahb_base + D2A_REG_P1ADDR, (unsigned long)vb->plane.mem_priv);
//            break;
//        default:;
//        }
//    }

    kdp_list_init(&fb->list);

    if (MIPI_CSI2RX_0 == ctx->id) {
        m_tid_csi2rx_array[0] = osThreadNew(csi2rx_thread_0, (void*)ctx, NULL);
    }
    else if (MIPI_CSI2RX_1 == ctx->id) {
        m_tid_csi2rx_array[1] = osThreadNew(csi2rx_thread_1, (void*)ctx, NULL);
    }    
    
    return 0;
}

static void kdp520_videobuf_queue(struct video_buf *vb)
{
    struct sys_camera_device *cam_d = sys_camera_from_vb2q(vb->vb_queue);
    struct sys_camera_host *cam_h = to_sys_camera_host(cam_d->parent_pin);
    struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context *)cam_h->priv;
    struct kdp520_fb *fb = to_kdp520_vb(vb);

    spin_lock_irq(&ctx->lock);

    kdp_list_add_tail(&ctx->fb_list, &fb->list);

    if (!ctx->active) {
        ctx->active = fb;
        //DSG("<%s> ctx->active_fb_idx=%d", __func__, vb->index);
    }
    
    spin_unlock_irq(&ctx->lock);
}

static int kdp520_start_streaming(struct video_buf_queue *vq)
{
    struct sys_camera_device *cam_d = sys_camera_from_vb2q(vq);
    struct sys_camera_host *cam_h = to_sys_camera_host(cam_d->parent_pin);
    struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context *)cam_h->priv;

    DSG("[%s]", __func__);
    
    //PHY Protocol interface turn on/off
    if (MIPI_CSI2RX_0 == ctx->id) {
    
        CSIRX_REG_PECR_SET_PEC(ctx->base + CSI2RX_REG_PECR, 0);
    #if IMGSRC_0_MIPILANE_NUM == IMAGE_MIPILANE_NUM_2
        CSIRX_REG_PECR_SET_PEC(ctx->base + CSI2RX_REG_PECR, BIT1 | BIT0);
    #elif IMGSRC_0_MIPILANE_NUM == IMAGE_MIPILANE_NUM_1
        CSIRX_REG_PECR_SET_PEC(ctx->base + CSI2RX_REG_PECR, BIT0);
    #endif

    #ifdef DEBUG_CSI2RX_REG
        DSG("11 CSI2RX_REG_VIDR=%x", inw(ctx->base +CSI2RX_REG_VIDR));
        DSG("11 CSI2RX_REG_DIDR=%x", inw(ctx->base +CSI2RX_REG_DIDR));
        DSG("11 CSI2RX_REG_CR=%x", inw(ctx->base +CSI2RX_REG_CR));
        DSG("11 CSI2RX_REG_VSCR=%x", inw(ctx->base +CSI2RX_REG_VSCR));
        DSG("11 CSI2RX_REG_ECR=%x", inw(ctx->base +CSI2RX_REG_ECR));
        DSG("11 CSI2RX_REG_TCNR=%x", inw(ctx->base +CSI2RX_REG_TCNR));
        DSG("11 CSI2RX_REG_HRTVR=%x", inw(ctx->base +CSI2RX_REG_HRTVR));
        DSG("11 CSI2RX_REG_FIUR=%x", inw(ctx->base +CSI2RX_REG_FIUR));
        DSG("11 CSI2RX_REG_ITR=%x", inw(ctx->base +CSI2RX_REG_ITR));
        DSG("11 CSI2RX_REG_VSTR0=%x", inw(ctx->base +CSI2RX_REG_VSTR0));
        DSG("11 CSI2RX_REG_HSTR0=%x", inw(ctx->base +CSI2RX_REG_HSTR0));
        DSG("11 CSI2RX_REG_VSTR1=%x", inw(ctx->base +CSI2RX_REG_VSTR1));
        DSG("11 CSI2RX_REG_HSTR1=%x", inw(ctx->base +CSI2RX_REG_HSTR1));
        DSG("11 CSI2RX_REG_VSTR2=%x", inw(ctx->base +CSI2RX_REG_VSTR2));
        DSG("11 CSI2RX_REG_HSTR2=%x", inw(ctx->base +CSI2RX_REG_HSTR2));
        DSG("11 CSI2RX_REG_VSTR3=%x", inw(ctx->base +CSI2RX_REG_VSTR3));
        DSG("11 CSI2RX_REG_HSTR3=%x", inw(ctx->base +CSI2RX_REG_HSTR3));
        DSG("11 CSI2RX_REG_MCR=%x", inw(ctx->base +CSI2RX_REG_MCR));
        DSG("11 CSI2RX_REG_VSTER=%x", inw(ctx->base +CSI2RX_REG_VSTER));
        DSG("11 CSI2RX_REG_HPNR=%x", inw(ctx->base +CSI2RX_REG_HPNR));

        DSG("11 CSI2RX_REG_PECR=%x", inw(ctx->base +CSI2RX_REG_PECR));
        DSG("11 CSI2RX_REG_DLMR=%x", inw(ctx->base +CSI2RX_REG_DLMR));
        DSG("11 CSI2RX_REG_CSIERR=%x", inw(ctx->base +CSI2RX_REG_CSIERR));
        DSG("11 CSI2RX_REG_INTSTS=%x", inw(ctx->base +CSI2RX_REG_INTSTS));
        DSG("11 CSI2RX_REG_ESR=%x", inw(ctx->base +CSI2RX_REG_ESR));

        DSG("11 CSI2RX_REG_DPISR=%x", inw(ctx->base +CSI2RX_REG_DPISR));
        DSG("11 CSI2RX_REG_INTER=%x", inw(ctx->base +CSI2RX_REG_INTER));
        DSG("11 CSI2RX_REG_FFR=%x", inw(ctx->base +CSI2RX_REG_FFR));
        DSG("11 CSI2RX_REG_DPCMR=%x", inw(ctx->base +CSI2RX_REG_DPCMR));
        DSG("11 CSI2RX_REG_FRR=%x", inw(ctx->base +CSI2RX_REG_FRR));

        DSG("11 CSI2RX_REG_PFTR=%x", inw(ctx->base +CSI2RX_REG_PFTR));
        DSG("11 CSI2RX_REG_PUDTR=%x", inw(ctx->base +CSI2RX_REG_PUDTR));
        DSG("11 CSI2RX_REG_FRCR=%x", inw(ctx->base +CSI2RX_REG_FRCR));
        DSG("11 CSI2RX_REG_FNR=%x", inw(ctx->base +CSI2RX_REG_FNR));
        DSG("11 CSI2RX_REG_BPGLR=%x", inw(ctx->base +CSI2RX_REG_BPGLR));     
        DSG("11 CSI2RX_REG_BPGHR=%x", inw(ctx->base +CSI2RX_REG_BPGHR));     
        

        DSG("11 D2A_REG_CTRL=%x", inw(ctx->dpi2ahb_base +D2A_REG_CTRL));
        DSG("11 D2A_REG_FNC=%x", inw(ctx->dpi2ahb_base +D2A_REG_FNC));
        DSG("11 D2A_REG_P0ADDR=%x", inw(ctx->dpi2ahb_base +D2A_REG_P0ADDR));
        DSG("11 D2A_REG_P1ADDR=%x", inw(ctx->dpi2ahb_base +D2A_REG_P1ADDR));
        DSG("11 D2A_REG_ICT=%x", inw(ctx->dpi2ahb_base +D2A_REG_ICT));
        DSG("11 D2A_REG_IS=%x", inw(ctx->dpi2ahb_base +D2A_REG_IS));
        DSG("11 D2A_REG_PT=%x", inw(ctx->dpi2ahb_base +D2A_REG_PT));
        DSG("11 D2A_REG_FIU=%x", inw(ctx->dpi2ahb_base +D2A_REG_FIU));

    #endif

        NVIC_EnableIRQ((IRQn_Type)D2A_FTDPI2AHB_IRQ);
    } 
    else if (MIPI_CSI2RX_1 == ctx->id) {

        CSIRX_REG_PECR_SET_PEC(ctx->base + CSI2RX_REG_PECR, 0);
    #if IMGSRC_1_MIPILANE_NUM == IMAGE_MIPILANE_NUM_2
        CSIRX_REG_PECR_SET_PEC(ctx->base + CSI2RX_REG_PECR, BIT1 | BIT0);
    #elif IMGSRC_1_MIPILANE_NUM == IMAGE_MIPILANE_NUM_1
        CSIRX_REG_PECR_SET_PEC(ctx->base + CSI2RX_REG_PECR, BIT0);
    #endif

    #ifdef DEBUG_CSI2RX_REG
        SCU_EXTREG_PRINT_ALL();
        DSG("22 CSI2RX_REG_VIDR=%x", inw(ctx->base +CSI2RX_REG_VIDR));
        DSG("22 CSI2RX_REG_DIDR=%x", inw(ctx->base +CSI2RX_REG_DIDR));
        DSG("22 CSI2RX_REG_CR=%x", inw(ctx->base +CSI2RX_REG_CR));
        DSG("22 CSI2RX_REG_VSCR=%x", inw(ctx->base +CSI2RX_REG_VSCR));
        DSG("22 CSI2RX_REG_ECR=%x", inw(ctx->base +CSI2RX_REG_ECR));
        DSG("22 CSI2RX_REG_TCNR=%x", inw(ctx->base +CSI2RX_REG_TCNR));
        DSG("22 CSI2RX_REG_HRTVR=%x", inw(ctx->base +CSI2RX_REG_HRTVR));
        DSG("22 CSI2RX_REG_FIUR=%x", inw(ctx->base +CSI2RX_REG_FIUR));
        DSG("22 CSI2RX_REG_ITR=%x", inw(ctx->base +CSI2RX_REG_ITR));
        DSG("22 CSI2RX_REG_VSTR0=%x", inw(ctx->base +CSI2RX_REG_VSTR0));
        DSG("22 CSI2RX_REG_HSTR0=%x", inw(ctx->base +CSI2RX_REG_HSTR0));
        DSG("22 CSI2RX_REG_VSTR1=%x", inw(ctx->base +CSI2RX_REG_VSTR1));
        DSG("22 CSI2RX_REG_HSTR1=%x", inw(ctx->base +CSI2RX_REG_HSTR1));
        DSG("22 CSI2RX_REG_VSTR2=%x", inw(ctx->base +CSI2RX_REG_VSTR2));
        DSG("22 CSI2RX_REG_HSTR2=%x", inw(ctx->base +CSI2RX_REG_HSTR2));
        DSG("22 CSI2RX_REG_VSTR3=%x", inw(ctx->base +CSI2RX_REG_VSTR3));
        DSG("22 CSI2RX_REG_HSTR3=%x", inw(ctx->base +CSI2RX_REG_HSTR3));
        DSG("22 CSI2RX_REG_MCR=%x", inw(ctx->base +CSI2RX_REG_MCR));
        DSG("22 CSI2RX_REG_VSTER=%x", inw(ctx->base +CSI2RX_REG_VSTER));
        DSG("22 CSI2RX_REG_HPNR=%x", inw(ctx->base +CSI2RX_REG_HPNR));

        DSG("22 CSI2RX_REG_PECR=%x", inw(ctx->base +CSI2RX_REG_PECR));
        DSG("22 CSI2RX_REG_DLMR=%x", inw(ctx->base +CSI2RX_REG_DLMR));
        DSG("22 CSI2RX_REG_CSIERR=%x", inw(ctx->base +CSI2RX_REG_CSIERR));
        DSG("22 CSI2RX_REG_INTSTS=%x", inw(ctx->base +CSI2RX_REG_INTSTS));
        DSG("22 CSI2RX_REG_ESR=%x", inw(ctx->base +CSI2RX_REG_ESR));

        DSG("22 CSI2RX_REG_DPISR=%x", inw(ctx->base +CSI2RX_REG_DPISR));
        DSG("22 CSI2RX_REG_INTER=%x", inw(ctx->base +CSI2RX_REG_INTER));
        DSG("22 CSI2RX_REG_FFR=%x", inw(ctx->base +CSI2RX_REG_FFR));
        DSG("22 CSI2RX_REG_DPCMR=%x", inw(ctx->base +CSI2RX_REG_DPCMR));
        DSG("22 CSI2RX_REG_FRR=%x", inw(ctx->base +CSI2RX_REG_FRR));

        DSG("22 CSI2RX_REG_PFTR=%x", inw(ctx->base +CSI2RX_REG_PFTR));
        DSG("22 CSI2RX_REG_PUDTR=%x", inw(ctx->base +CSI2RX_REG_PUDTR));
        DSG("22 CSI2RX_REG_FRCR=%x", inw(ctx->base +CSI2RX_REG_FRCR));
        DSG("22 CSI2RX_REG_FNR=%x", inw(ctx->base +CSI2RX_REG_FNR));
        DSG("22 CSI2RX_REG_BPGLR=%x", inw(ctx->base +CSI2RX_REG_BPGLR));
        DSG("22 CSI2RX_REG_BPGHR=%x", inw(ctx->base +CSI2RX_REG_BPGHR));
    #endif

        NVIC_EnableIRQ((IRQn_Type)D2A_FTDPI2AHB_1_IRQ);
    }


    //osThreadFlagsSet(m_tid_csi2rx_array, FLAG_CSI2RX_START);

    return 0;
}

static void kdp520_stop_streaming(struct video_buf_queue *vq)
{
    struct sys_camera_device *cam_d = sys_camera_from_vb2q(vq);
    struct sys_camera_host *cam_h = to_sys_camera_host(cam_d->parent_pin);
    struct kdp520_csi2rx_context *ctx = (struct kdp520_csi2rx_context *)cam_h->priv;
    u32 val;

    DSG("<%s>", __func__);

    //PHY Protocol interface turn on/off
    if (MIPI_CSI2RX_0 == ctx->id) {
        val = inw(ctx->base + CSI2RX_REG_PECR);
        outw(ctx->base + CSI2RX_REG_PECR, (val & 0xFFFFFF00));
    } 
    else if (MIPI_CSI2RX_1 == ctx->id) {
        val = inw(ctx->base + CSI2RX_REG_PECR);
        outw(ctx->base + CSI2RX_REG_PECR, (val & 0xFFFFFF00));
    }    
}

void kdp520_wait_prepare(struct video_buf_queue *vq)
{
    struct sys_camera_device *cam_d = sys_camera_from_vb2q(vq);
    struct sys_camera_host *cam_h = to_sys_camera_host(cam_d->parent_pin);

    mutex_unlock(&vq->lock);
}

void kdp520_wait_finish(struct video_buf_queue *vq)
{
    mutex_lock(&vq->lock);
}

// static void kdp520_videobuf_release(struct video_buf *vb)
// {

// }

static struct videobuf_ops kdp520_videobuf_ops = {
    .queue_setup    = kdp520_videobuf_setup,
    .buf_init       = kdp520_videobuf_init,        
    .buf_queue      = kdp520_videobuf_queue,
    .start_streaming= kdp520_start_streaming,
    .stop_streaming = kdp520_stop_streaming,    
    .wait_prepare   = kdp520_wait_prepare,
    .wait_finish    = kdp520_wait_finish,
    //.buf_cleanup    = kdp520_videobuf_release,
};

const struct videobuf_mem_ops kdp520_videobuf_mem_ops = {
    .alloc = dm_alloc,
};

static int kdp520_csi2rx_init_videobuf(struct video_buf_queue *vq,
        struct sys_camera_device *cam_d)
{
    struct sys_camera_host *cam_h = to_sys_camera_host(cam_d->parent_pin);

    vq->vb_ops = &kdp520_videobuf_ops;
    vq->vb_mem_ops = &kdp520_videobuf_mem_ops;
    vq->buf_struct_size = sizeof(struct kdp520_fb);

    kdp_list_init(&vq->queued_list);
    kdp_list_init(&vq->done_list);

    mutex_init(&vq->lock);

    return 0;
}

static struct sys_camera_host_ops kdp520_sys_camera_host_ops = {
    .clock_start    = kdp520_csi2rx_clock_start,
    .clock_stop     = kdp520_csi2rx_clock_stop,
    .reset          = kdp520_csi2rx_reset,    
    .querycap       = kdp520_csi2rx_querycap,
    .set_fmt        = kdp520_csi2rx_set_fmt,
    .set_bus_param  = kdp520_csi2rx_set_bus_param,	
    .init_videobuf  = kdp520_csi2rx_init_videobuf, 
};

static int kdp520_csi2rx_probe(struct core_device *core_d)
{
    struct kdp520_csi2rx_context *ctx;
    struct mm_struct *mm = get_mm_struct();    
    int err;
    
    DSG("[%s]", __func__);

    ctx = calloc(1, sizeof(*ctx));
    if (!ctx) {
        //DSG("<%s> Could not allocate pdev", __func__);
        return -KDP_FRAMEWORK_ERRNO_NOMEM;
    }

    kdp_list_init(&ctx->fb_list);

    if (0 == m_csi2rx_driver_inst_num)
        g_csi2rx0_ctx = (struct kdp520_csi2rx_context_lite *)ctx;
    else if (1 == m_csi2rx_driver_inst_num)
        g_csi2rx1_ctx = (struct kdp520_csi2rx_context_lite *)ctx;
    
    //ctx->inst_idx = m_csi2rx_driver_inst_num;
    if (0 == m_csi2rx_driver_inst_num) {
    #if V2K_ENABLE_TYPE == V2K_ENABLE_HMX2056
        ctx->ai_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_TO_NCPU);
        ctx->ai_mem_addr_tmp    = (DDR_MEM_BASE + DDR_OFFSET_RGB_TO_REST);
        ctx->vw_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_PRIMARY);
        ctx->ov_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_OVERLAY);
        ctx->sid = SENSOR_TYPE_HMX2056;
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_OV9286
        ctx->ai_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_NCPU);
        ctx->ai_mem_addr_tmp    = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_REST);
        ctx->vw_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_PRIMARY);
        ctx->ov_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_OVERLAY);
        ctx->sid = SENSOR_TYPE_OV9286;
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_HMX2056_OV9286
        ctx->ai_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_TO_NCPU);
        ctx->ai_mem_addr_tmp    = (DDR_MEM_BASE + DDR_OFFSET_RGB_TO_REST);
        ctx->vw_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_PRIMARY);
        ctx->ov_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_OVERLAY);
        ctx->sid = SENSOR_TYPE_HMX2056;
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_HMXRICA
        ctx->ai_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_NCPU);
        ctx->ai_mem_addr_tmp    = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_REST);
        ctx->vw_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_PRIMARY);
        ctx->ov_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_OVERLAY);
        ctx->sid = SENSOR_TYPE_HMXRICA;
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_GC2145
        ctx->ai_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_TO_NCPU);
        ctx->ai_mem_addr_tmp    = (DDR_MEM_BASE + DDR_OFFSET_RGB_TO_REST);
        ctx->vw_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_PRIMARY);
        ctx->ov_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_OVERLAY);
        ctx->sid = SENSOR_TYPE_GC2145;
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_SC132GS
        ctx->ai_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_NCPU);
        ctx->ai_mem_addr_tmp    = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_REST);
        ctx->vw_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_PRIMARY);
        ctx->ov_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_OVERLAY);
        ctx->sid = SENSOR_TYPE_SC132GS;
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_GC2145_SC132GS
        ctx->ai_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_TO_NCPU);
        ctx->ai_mem_addr_tmp    = (DDR_MEM_BASE + DDR_OFFSET_RGB_TO_REST);
        ctx->vw_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_PRIMARY);
        ctx->ov_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_OVERLAY);
        ctx->sid = SENSOR_TYPE_GC2145;
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_SC132GS_GC2145
        ctx->ai_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_NCPU);
        ctx->ai_mem_addr_tmp    = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_REST);
        ctx->vw_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_PRIMARY);
        ctx->ov_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_OVERLAY);
        ctx->sid = SENSOR_TYPE_SC132GS;
    #endif
    }
    else if (1 == m_csi2rx_driver_inst_num) {
    #if V2K_ENABLE_TYPE == V2K_ENABLE_HMX2056_OV9286
        ctx->ai_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_NCPU);
        ctx->ai_mem_addr_tmp    = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_REST);
        ctx->vw_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_PRIMARY);
        ctx->ov_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_OVERLAY);
        ctx->sid = SENSOR_TYPE_OV9286;
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_GC2145_SC132GS
        ctx->ai_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_NCPU);
        ctx->ai_mem_addr_tmp    = (DDR_MEM_BASE + DDR_OFFSET_NIR_TO_REST);
        ctx->vw_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_PRIMARY);
        ctx->ov_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_NIR_OVERLAY);
        ctx->sid = SENSOR_TYPE_SC132GS;
    #elif V2K_ENABLE_TYPE == V2K_ENABLE_SC132GS_GC2145
        ctx->ai_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_TO_NCPU);
        ctx->ai_mem_addr_tmp    = (DDR_MEM_BASE + DDR_OFFSET_RGB_TO_REST);
        ctx->vw_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_PRIMARY);
        ctx->ov_mem_addr        = (DDR_MEM_BASE + DDR_OFFSET_RGB_OVERLAY);
        ctx->sid = SENSOR_TYPE_GC2145;
    #endif
    }

    DSG("ctx->ai_mem_addr=0x%x", ctx->ai_mem_addr);
    DSG("ctx->ai_mem_addr_tmp=0x%x", ctx->ai_mem_addr_tmp);
    DSG("ctx->vw_mem_addr=0x%x", ctx->vw_mem_addr);
    DSG("ctx->ov_mem_addr=0x%x", ctx->ov_mem_addr);

    ctx->base = driver_get_ioport_setting(core_d, IORESOURCE_MEM)->start;
    ctx->dpi2ahb_base = driver_get_ioport_setting(core_d, IORESOURCE_BUS)->start;
    ctx->field = V2K_FIELD_NONE;
    
    if ((u32)(ctx->base) == CSIRX_0_BASE)
    {
        ctx->id = MIPI_CSI2RX_0;
    }
    else if ((u32)(ctx->base) == CSIRX_1_BASE)
    {
        ctx->id = MIPI_CSI2RX_1;
    }

    ctx->cam_h.priv = ctx;
    ctx->cam_h.pin_obj.pin_ctx = &core_d->pin_ctx;
    ctx->cam_h.host_id = core_d->uuid;
    DSG("[%s] ctx->cam_h.host_id = (%s) %d", __func__, core_d->name, ctx->cam_h.host_id);
    ctx->cam_h.ops = &kdp520_sys_camera_host_ops;
    {
        struct videobuf_dm_ctx *p = calloc(1, sizeof(struct videobuf_dm_ctx));
        // To do ...
        p->seq_num = 0;
        if (MIPI_CSI2RX_0 == ctx->id) {
            p->vir_start_addr = mm->mmap->vm_start + mm->csirx0_mm_user;
            p->phy_start_addr = mm->mmap->vm_start + mm->csirx0_mm_phy;
            DSG("(%s) MIPI_CSI2RX_0 phy_start_addr=%x", __func__, p->phy_start_addr);
        }
        else if (MIPI_CSI2RX_1 == ctx->id) {
            p->vir_start_addr = mm->mmap->vm_start + mm->csirx1_mm_user;
            p->phy_start_addr = mm->mmap->vm_start + mm->csirx1_mm_phy;
            DSG("(%s) MIPI_CSI2RX_1 phy_start_addr=%x", __func__, p->phy_start_addr);                     
        }
        p->allocated_length = 0;

        ctx->alloc_ctx = (void*)p;
    }
    
    framework_drv_set_drvdata(&core_d->pin_ctx, (void*)ctx);

    err = sys_camera_host_register(&ctx->cam_h);
    if (err) {
        goto exit_csi2_unregister;
    }

    ++m_csi2rx_driver_inst_num;

    return 0;

exit_csi2_unregister:
    sys_camera_host_unregister(&ctx->cam_h);

    return err;
}

static int kdp520_csi2rx_remove(struct core_device *core_d)
{
    struct sys_camera_host *sch = to_sys_camera_host(&core_d->pin_ctx);

    sys_camera_host_unregister(sch);

    return 0;
}

extern struct core_device kdp520_csi2rx0;
static struct core_driver kdp520_csi2rx0_driver = {
    .driver     = {
        .name   = "kdp520_csi2rx0",
    },
    .probe      = kdp520_csi2rx_probe,
    .remove     = kdp520_csi2rx_remove,
    .core_dev   = &kdp520_csi2rx0,
    
    .power_mgr  = {
        .power      = kdp520_csi2rx_power,
        .suspend    = NULL,
        .resume     = NULL,
    }
};
KDP_CORE_DRIVER_SETUP(kdp520_csi2rx0_driver, 5);

#if (V2K_ENABLE_TYPE == V2K_ENABLE_HMX2056_OV9286) || \
    (V2K_ENABLE_TYPE == V2K_ENABLE_GC2145_SC132GS) || \
    (V2K_ENABLE_TYPE == V2K_ENABLE_SC132GS_GC2145)
extern struct core_device kdp520_csi2rx1;
static struct core_driver kdp520_csi2rx1_driver = {
    .driver     = {
        .name   = "kdp520_csi2rx1",
    },
    .probe      = kdp520_csi2rx_probe,
    .remove     = kdp520_csi2rx_remove,
    .core_dev   = &kdp520_csi2rx1,
    
    .power_mgr  = {
        .power      = kdp520_csi2rx_power,
        .suspend    = NULL,
        .resume     = NULL,
    }    
};
KDP_CORE_DRIVER_SETUP(kdp520_csi2rx1_driver, 5);
#endif

#endif
#endif
