#include "cmsis_os2.h"
#include "fifoq.h"

#include "kmdw_memory.h"
#include "kmdw_console.h"

fifoQ_handles_t fifoQ_init(uint32_t num_queue, uint32_t data_buf_size, uint32_t result_buf_size)
{
    dual_fifo_t image_dfifo = dual_fifo__create(num_queue, data_buf_size);
    dual_fifo_t result_dfifo = dual_fifo__create(num_queue, result_buf_size);

    // reserve DDR memory for image and result buffers
    for (int i = 0; i < num_queue; i++)
    {
        uint32_t buf_addr;

        buf_addr = kmdw_ddr_reserve(data_buf_size);
        dual_fifo__put_free_buffer(image_dfifo, (const void *)&buf_addr, osWaitForever);

        buf_addr = kmdw_ddr_reserve(result_buf_size);
        dual_fifo__put_free_buffer(result_dfifo, (const void *)&buf_addr, osWaitForever);
    }

#define USBD_ENDPOINT_DATA_OUT 0x03 // FIXME, should sync with usb_com.c
#define USBD_ENDPOINT_DATA_IN 0x84  // FIXME, should sync with usb_com.c

    // create threads for handling data between usb and fifo queue
    fifoQ_usb_init(image_dfifo, result_dfifo, USBD_ENDPOINT_DATA_IN, USBD_ENDPOINT_DATA_OUT);

    fifoQ_handles_t fq = {image_dfifo, result_dfifo};

    return fq;
}
