/**
 * Copyright (C) Kneron Inc. All right reserved
 *
 * @file kapp_dme.h
 * @brief Kneron Application Dynamic Model Execution APIs
 */

#ifndef __KAPP_DME_H_
#define __KAPP_DME_H_

#include <stdint.h>
#include "kapp.h"
#include "ipc.h"

#define MSG_DME_BAD_SSID   248  // unknown session id
#define MSG_DME_DATA_FAIL  253  // data download failed
#define MSG_DME_NO_CONFIG  244  // DME config not issued previously
#define MSG_DME_Q_FAILED   243  // DME I/O queue internal error
#define MSG_DME_Q_FULL     242  // DME input queue full
#define MSG_DME_BAD_MODEL  241  // DME model id error

struct kdp_dme_cfg {
    int32_t model_id;
    int32_t output_num;
    struct kdp_img_conf_s   img_conf;
    struct kdp_crop_box_s   crop_box;
    struct kdp_pad_value_s  pad_values;
    float                   ext_param[MAX_PARAMS_LEN];
};

struct kdp_dme_conf_tbl_s {
    int32_t model_count;
    struct  kdp_dme_cfg dme_config[];
};

/**
 * @brief generate img config from DME config
 * @param dme_conf DME config
 * @param img_buf_idx active image buffer index
 * @return img config
 */
struct kdp_img_cfg kapp_dme_gen_img_cfg(const struct kdp_dme_cfg *dme_conf, int img_buf_idx);

/**
 * @brief do dynamic model execution (DME) to get detection output or raw output
 * @param p_out_p dynamic model execution result
 * @param img_conf image configuration
 * @return OK or error code
 */
int32_t kapp_dme(dme_res* p_out_p, struct kdp_dme_cfg *img_conf);

#endif
