/*
 * Kneron Host Communication driver
 *
 * Copyright (C) 2018 Kneron, Inc. All rights reserved.
 *
 */
#include <stdio.h>
#include <stdint.h>
#include "base.h"
#include "com.h"
#include "com_err.h"
#include "msg.h"
#include "kmdw_console.h"
#include "io.h"
#include "kcomm.h"
#include "kmdw_ota.h"
#include "kdrv_clock.h"
#include "kdrv_power.h"

#define DEF_LOG_CATEG  "kcomm"

static void delay_after_fw_update()
{
    kdrv_delay_us(50*1000);
    kdrv_power_sw_reset();
}

static void handle_update_fw(RspPram *rsp, u32 tgt_id)
{
    dlog("tgt_id [%u]", tgt_id);

    int ret = -1;

    switch(tgt_id) {
    case UPDATE_MODULE_SCPU:
        ret = kmdw_ota_update_scpu();
        rsp->error = ret;
        rsp->param2 = UPDATE_MODULE_SCPU;
        kcomm_send_rsp(CMD_UPDATE_FW, NULL, 0);
        break;
    case UPDATE_MODULE_NCPU:
        ret = kmdw_ota_update_ncpu();
        rsp->error = ret;
        rsp->param2 = UPDATE_MODULE_NCPU;
        kcomm_send_rsp(CMD_UPDATE_FW, NULL, 0);
        break;
    default:
        break;
    }

    if (ret == SUCCESS) {
        delay_after_fw_update();
    }
}

static void handle_update_model(RspPram *rsp, u32 info_size, u32 model_size)
{
    dlog("info_size [%u], model_size [%u]", info_size, model_size);

    if (info_size == 0)
        return;

    int ret = kmdw_ota_update_model(info_size, model_size);
    rsp->error = ret;
    rsp->param2 = info_size; /* not used */
    kcomm_send_rsp(CMD_UPDATE_MODEL, NULL, 0);

    if (ret == SUCCESS) {
        delay_after_fw_update();
    }
}

static int handle_ota_cmd(MsgHdr *hdr, CmdPram *args, RspPram *rsp, void *data)
{
    u16 cmd = hdr->cmd;
    u32 arg1 = args->param1;
    u32 arg2 = args->param2;

    switch (cmd) {
    case CMD_UPDATE_FW:
        handle_update_fw(rsp, arg1);
        break;
    case CMD_UPDATE_MODEL:
        handle_update_model(rsp, arg1, arg2);
        break;
    default:
        return -1;
    }

    return 0;
}

static void init_cmd_handler()
{
    kmdw_ota_init(msg_rbuf, kcomm_read);
}

/* ############################
 * ##    public functions    ##
 * ############################ */

void kcomm_enable_ota_cmds(void)
{
    init_cmd_handler();
    kcomm_reg_cmd_handler(handle_ota_cmd);
}
