/*
 * Kneron SDK Main Init driver
 *
 * Copyright (C) 2020 Kneron, Inc. All rights reserved.
 *
 */

#include <stdio.h>
#include <string.h>

#include "kneron_mozart.h"                // Device header
#include "kneron_mozart_ext.h"
#include "kdrv_ddr.h"
#include "kmdw_model.h"
#include "kdrv_system.h"
#include "kdrv_scu.h"
#include "kdrv_clock.h"
#include "kdev_flash.h"
#include "power_manager.h"
#include "system_ncpu.h"
#include "kmdw_memory.h"     //for ddr_malloc
#include "kdp_ddr_table.h"  //for default ddr table
#include "kdrv_mpu.h"
#include "version.h"
#include "kmdw_console.h"
#include "pinmux.h"
#include "kmdw_camera.h"
#include "kmdw_display.h"
#include "kdrv_i2c.h"
#include "kdrv_uart.h"
#include "kdp_system.h"
#include "kmdw_ota.h"

uint32_t kdp_sys_get_key_status(void);
void main_load_ncpu(void);

extern u32 bootup_status;
extern u32 warm_boot;

//=================================================================
__weak void kdrv_spif_set_commands(uint32_t cmd0, uint32_t cmd1, uint32_t cmd2, uint32_t cmd3)
{
}

__weak void kdrv_spif_wait_command_complete(void)
{
}

__weak void kdrv_spif_check_quad_status_till_ready(void)
{
}

__weak void kdrv_spif_memxfer_initialize(uint8_t flash_mode, uint8_t mem_mode)
{
}

__weak uint32_t kdrv_spif_rxfifo_depth(void)
{
    return 0;
}

__weak uint32_t kdrv_spif_txfifo_depth(void)
{
    return 0;
}

__weak void kdrv_spif_wait_rx_full(void)
{
}

__weak void kdrv_spif_wait_tx_empty(void)
{
}

__weak void kdev_flash_4Bytes_ctrl(uint8_t  enable)
{
}

__weak void kdev_flash_read_flash_id(void)
{
}

__weak void kdev_flash_write_control(uint8_t  enable)
{
}

__weak uint32_t  kdev_flash_probe(spi_flash_t *flash)
{
	return KDEV_STATUS_OK;
}

__weak kdev_status_t kdev_flash_initialize(void)
{
	return KDEV_STATUS_OK;
}

__weak kdev_status_t kdev_flash_uninitialize(void)
{
	return KDEV_STATUS_OK;
}

__weak kdev_status_t kdev_flash_power_control(ARM_POWER_STATE state)
{
	return KDEV_STATUS_OK;
}

__weak kdev_status_t kdev_flash_readdata(uint32_t addr, void *data, uint32_t cnt)
{
	return KDEV_STATUS_OK;
}

__weak void kdev_flash_read(uint8_t  type, uint32_t  offset, uint32_t  len, void *buf)
{
}

__weak kdev_status_t kdev_flash_programdata (uint32_t addr, const void *data, uint32_t cnt)
{
	return KDEV_STATUS_OK;
}

__weak kdev_status_t kdev_flash_programdata_memxfer(uint32_t addr, const void *data, uint32_t cnt)
{
	return KDEV_STATUS_OK;
}

__weak kdev_status_t kdev_flash_erase_sector(uint32_t addr)
{
	return KDEV_STATUS_OK;
}

__weak kdev_status_t kdev_flash_erase_multi_sector(uint32_t start_addr, uint32_t end_addr)
{
	return KDEV_STATUS_OK;
}

__weak kdev_status_t kdev_flash_erase_chip(void)
{
	return KDEV_STATUS_OK;
}

__weak kdev_flash_status_t kdev_flash_get_status(void)
{
	kdev_flash_status_t status;
	return status;
}

__weak kdev_status_t kdev_flash_get_info(void)
{
	return KDEV_STATUS_OK;
}

__weak void kdev_flash_64kErase(uint32_t  offset)
{
}

__weak void kdev_flash_4kErase(uint32_t  offset)
{
}

static void print_key_info(void)
{
    uint32_t keystatus = kdp_sys_get_key_status();

    DSG("\n#########################################");
    if(keystatus == 0) {
        DSG("## With key  ##");
    } else if(keystatus == 1) {
        DSG("## Without key  ##");
    } else if(keystatus == 0xff) {
        DSG("## Fail to read eFuse status  ##");
    }
    DSG("#########################################\n");
}

static void print_versions(int host_mode)
{
    uint32_t ver;
	int a, x, y, z, b;

    if (host_mode) {
        DSG("=== Versions (host mode) ===");
    } else {
        DSG("=== Versions (companion mode) ===");
    }

    ver = kdp_sys_get_spl_version();
    a = ver & 0xFF;
    x = (ver >> 8) & 0xFF;
    y = (ver >> 16) & 0xFF;
    z = (ver >> 24) & 0xFF;
    b = kdp_sys_get_spl_build();
    DSG("  SPL: %d.%d.%d.%d-build.%d", a, x, y, z, b);

    ver = kdp_sys_get_ncpu_version();
    a = ver & 0xFF;
    x = (ver >> 8) & 0xFF;
    y = (ver >> 16) & 0xFF;
    z = (ver >> 24) & 0xFF;
    b = kdp_sys_get_ncpu_build();
    DSG(" NCPU: %d.%d.%d.%d-build.%d", a, x, y, z, b);

    DSG(" SCPU: %d.%d.%d.%d-build.%d", 
                   g_fw_misc.architecture,
                   g_fw_misc.major,
                   g_fw_misc.minor,
                   g_fw_misc.update,
                   g_fw_misc.build);
}

static void print_reset_info(void)
{
    if (warm_boot) {
        DSG("  -> Bootup Status: 0x%x (warm boot) <-", bootup_status);
    } else {
        DSG("  -> Bootup Status: 0x%x <-", bootup_status);
    }

    if (bootup_status & SCU_REG_BTUP_STS_HWR)
        DSG("                  : Hardware Reset");
    if (bootup_status & SCU_REG_BTUP_STS_WDR)
        DSG("                  : Watchdog Reset");
    if (bootup_status & SCU_REG_BTUP_STS_SMR)
        DSG("                  : Dormant Reset");
    if (bootup_status & SCU_REG_BTUP_STS_PMR)
        DSG("                  : Poweroff Reset");
    if (bootup_status & SCU_REG_BTUP_STS_PWRBTN_STS)
        DSG("                  : Power Button Reset");
    if (bootup_status & SCU_REG_BTUP_STS_RTC_BTUPTS)
        DSG("                  : RTC Alarm Reset");
    if (bootup_status & SCU_REG_BTUP_STS_PMR1)
        DSG("                  : Power Mode 1 Reset");
    if (bootup_status & SCU_REG_BTUP_STS_PMR2)
        DSG("                  : Power Mode 2 Reset");
}

static void info(int host_mode)
{
    uint32_t rom_table = inw(CPU_ID_STS_ADDR);
    if (CPU_ID_NCPU == rom_table) {
        DSG("!!! Wrong ROM Table. Please check if fw_scpu.bin is correct!!!");
    }

    print_key_info();

    print_reset_info();
}

/**
 * @brief main_init, main init function
 */
void main_init(int host_mode)
{
    kdrv_system_init();
    kdrv_system_init_ncpu();
    kdrv_ddr_system_init(DDR_INIT_ALL);

    if (host_mode) {
        pinmux_init();
    }

    kmdw_ddr_init(KDP_DDR_HEAP_HEAD_FOR_MALLOC+1, KDP_DDR_MODEL_RESERVED_END);
    kdrv_uart_initialize();
    kmdw_console_init(UART0_DEV);   // for log

    info(host_mode);
    power_manager_init();
    kmdw_model_init();

    if (host_mode) {
        kdrv_i2c_initialize(KDRV_I2C_CTRL_0, KDRV_I2C_SPEED_400K);
        kmdw_camera_init();
        kmdw_display_initialize();
    } else {
        kdrv_mpu_config();
    }

    main_load_ncpu();
    print_versions(host_mode);
}

void main_load_ncpu(void)
{
    /* load & launch ncpu fw from flash */
    uint32_t ncpu_sum32 = load_ncpu_fw(0);
    if(ncpu_sum32 != 0) {
        /* scpu image has been checked on the SPL */
        DSG("\n !!!! WARNING !!!! NCPU FW image check failure...(wrong sum32 0x%08x)\n", ncpu_sum32);
    }
    kdrv_delay_us(2000); /* this delay used to isolate init log message for scpu & ncpu */
}
