/*
 * Kneron Main Entry driver
 *
 * Copyright (C) 2020 Kneron, Inc. All rights reserved.
 *
 */

#include "kneron_mozart.h"                // Device header
#include "cmsis_os2.h"                    // ARM::CMSIS:RTOS2:Keil RTX5
#include "kcomm.h"
#include "kapp_ops.h"
#include "tiny_yolo_ops.h"

void main_init(int host_mode);

void kcomm_start(void)
{
    struct kapp_ops *ops;
    
    ops = tiny_yolo_get_ops();

    kcomm_enable_base_cmds();
    kcomm_enable_ota_cmds();
    kcomm_enable_dme_cmds();
    kcomm_enable_isi_cmds(ops);
    kcomm_init();
}

/**
 * @brief main, main function
 */
int main(void)
{
    SystemCoreClockUpdate();            // System Initialization
    osKernelInitialize();               // Initialize CMSIS-RTOS

    /* SDK main init */
    main_init(0);

    /* Application entry */
    kcomm_start();

    /* Start RTOS Kernel */
    if (osKernelGetState() == osKernelReady) {
        osKernelStart();
    }

    while(1) {
    }
}
