#include "board_kl520.h"
#include "console_example.h"
#include <string.h>
#include <stdlib.h>
#include <cmsis_os2.h>
#include "utility.h"
#include "kmdw_camera.h"
#include "kmdw_console.h"
#include "kmdw_memxfer.h"
#include "kmdw_display.h"
#include "kmdw_memory.h"
#include "kmdw_status.h"
#include "kdrv_i2c.h"
#include "kdrv_system.h"
#include "kdrv_power.h"
#include "kdrv_ddr.h"
#include "kdrv_clock.h"
#include "kdrv_status.h"
#include "system_ncpu.h"
#include "io.h"
#include "power_manager.h"
#include "kdp_ddr_table.h"

#define CAMERA_RGB_IDX              (MIPI_CAM_RGB)
#define CAMERA_NIR_IDX              (MIPI_CAM_NIR)
#define CAMERA_RGB_IMG_WIDTH        (640)
#define CAMERA_RGB_IMG_HEIGHT       (480)
#define CAMERA_NIR_IMG_WIDTH        (480)
#define CAMERA_NIR_IMG_HEIGHT       (640)
#define LCD_WIDTH                   LCDC_WIDTH
#define LCD_HEIGHT                  LCDC_HEIGHT

typedef void  (*console_cmd_func)(void);
struct console_cmd_info
{
    char *desc;
    console_cmd_func func;
    //void (*func)(void);
};

osThreadId_t tid_console_example = 0;
static bool g_load_fw_done = FALSE;
static bool display_inited = FALSE;
static int8_t display_cam = CAMERA_RGB_IDX;

struct camera_settings {
    uint32_t width;             /**< Camera image width*/
    uint32_t height;            /**< Camera image hieght*/
    uint32_t pixelformat;       /**< Camera image pixel format*/
} cam_settings[IMGSRC_NUM];


void sample_open_nir_led(void)
{
    dbg_msg("open nir led!\n");    
    kmdw_camera_led_switch(CAMERA_NIR_IDX, 1);
}

void sample_close_nir_led(void)
{
    dbg_msg("close nir led!\n");
    kmdw_camera_led_switch(CAMERA_NIR_IDX, 0);
}

static int _cmd_get_answer_int(const char *str)
{
    int ans;
    char buf[256];

    DSG_NOLF("%s", str);
    kmdw_console_echo_gets(buf, sizeof(buf));
    dbg_msg("");
    ans = atoi(strtok(buf, " \r\n\t"));
    return ans;
}

void sample_system_pll_setting(void)
{
    kdrv_system_init();
}

void sample_system_init_ncpu_clk(void)
{
    kdrv_system_init_ncpu();
}

void sample_system_init_ddr(void)
{
    kdrv_ddr_system_init(DDR_INIT_ALL);
    kmdw_ddr_init(KDP_DDR_HEAP_HEAD_FOR_MALLOC+1, KDP_DDR_MODEL_RESERVED_END);
}

void sample_load_ncpu_fw(void)
{
    /*No check CRC*/
    if(g_load_fw_done == TRUE) {
        info_msg("ncpu fw had been loaded\n");
        return;
    }
    load_ncpu_fw(-1);
    g_load_fw_done = TRUE;
}

void sample_init_power_mgr(void)
{
    power_manager_init();
}

void sample_init_mipi_camera(void)
{
    if(kdrv_i2c_initialize(KDRV_I2C_CTRL_0, KDRV_I2C_SPEED_400K) == KDRV_STATUS_OK) {
        if(kmdw_camera_init() != KDRV_STATUS_OK) {
            err_msg("Fails to initialize camera\n");
        }
    }
    else {
        err_msg("Fails to initialize I2C driver\n");
    }
}

void sample_camera_setting(uint8_t cam_idx, uint32_t width, uint32_t height, uint32_t pixelformat)
{
    cam_settings[cam_idx].width = width;
    cam_settings[cam_idx].height = height;
    cam_settings[cam_idx].pixelformat = pixelformat;
}

void sample_camera_open(void)
{
    int cam_idx;
    struct cam_format fmt;

    cam_idx = _cmd_get_answer_int("open camera, index >>");
    if(cam_idx == MIPI_CAM_RGB) {
        sample_camera_setting(cam_idx, IMGSRC_0_WIDTH, IMGSRC_0_HEIGHT, IMGSRC_0_FORMAT);
    }
    else if(cam_idx == MIPI_CAM_NIR) {
        sample_camera_setting(cam_idx, IMGSRC_1_WIDTH, IMGSRC_1_HEIGHT, IMGSRC_1_FORMAT);
    }
    else {
        return;
    }

    memset(&fmt, 0, sizeof(fmt));
    if(kmdw_camera_open(cam_idx) != 0) {
        err_msg("Fails to open camera\n");
        return;
    }        
    fmt.width = cam_settings[cam_idx].width;
    fmt.height = cam_settings[cam_idx].height;
    fmt.pixelformat = cam_settings[cam_idx].pixelformat;
    (void) kmdw_camera_set_frame_format(cam_idx, (struct cam_format *)&fmt);
    (void) kmdw_camera_get_frame_format(cam_idx, &fmt);
    (void) kmdw_camera_buffer_init(cam_idx);
    (void) kmdw_camera_start(cam_idx);
}

void sample_camera_close(void)
{
    int cam_idx;
    
    cam_idx = _cmd_get_answer_int("close camera, index >>");
    if((cam_idx != MIPI_CAM_RGB) || (cam_idx != MIPI_CAM_NIR)) {
        return;
    }

    if(kmdw_camera_stop(cam_idx) != KMDW_STATUS_OK) {
        err_msg("Fails to stop camera\n");
        return;
    }
    
    if (kmdw_camera_close(cam_idx)) {
        err_msg("Fails to close camera\n");
    }
}

void open_video_renderer(uint8_t cam_idx, uint32_t width, uint32_t height, uint32_t pixelformat)
{
    struct video_input_params params;

    if (display_inited == FALSE) {
        if(kmdw_display_initialize() == KMDW_STATUS_OK) {
            params.input_fmt = pixelformat;
            params.input_xres = width;
            params.input_yres = height;

            (void)kmdw_video_renderer_open(&params);
            (void)kmdw_video_renderer_set_camera(display_cam);
            (void)kmdw_video_renderer_buffer_initialize(&params);
            (void)kmdw_video_renderer_start();
            display_inited = TRUE;
        }
        else {
            err_msg("Fails to initialize display\n");
        }
    }
}

void sample_open_video_renderer(void)
{
    open_video_renderer(display_cam, LCD_WIDTH, LCD_HEIGHT, V2K_PIX_FMT_RGB565);   
}

void sample_close_video_renderer(void)
{
    if (display_inited == TRUE) {
        kmdw_video_renderer_stop();
        display_inited = FALSE;
    }
}

void sample_switch_video_renderer_src(void)
{
    display_cam = !display_cam;
    if(CAMERA_RGB_IDX == display_cam) {
        sample_close_video_renderer();
        open_video_renderer(display_cam, CAMERA_RGB_IMG_WIDTH, CAMERA_RGB_IMG_HEIGHT, V2K_PIX_FMT_RGB565);
    }
    else if(CAMERA_NIR_IDX == display_cam) {
        sample_close_video_renderer();
        open_video_renderer(display_cam, CAMERA_NIR_IMG_WIDTH, CAMERA_NIR_IMG_HEIGHT, V2K_PIX_FMT_RAW8);
    }
}

void sample_power_manager_reset(void)
{
    power_manager_reset();
}

void sample_power_manager_sleep(void)
{
    power_manager_sleep();
}

void sample_power_manager_deep_sleep(void)
{
    power_manager_deep_sleep();
}

void sample_power_manager_shutdown(void)
{
    power_manager_shutdown();
}

void sample_app_console_quit(void)
{
    DSG("bye bye !!");
    power_manager_shutdown();
}

struct console_cmd_info console_cmd_array[] = {
    /* System Initialization*/
    {"Turn on NPU/NCPU/DDR default power domain",       sample_system_pll_setting  },
    {"Initialize NPU/NCPU",                             sample_system_init_ncpu_clk  },
    {"Initialize DDR",                                  sample_system_init_ddr  },
    {"Load NCPU FW from FLASH to RAM",                  sample_load_ncpu_fw  },
    {"Initialize power manager",                        sample_init_power_mgr  },
    
    /*Operate sensors*/
    {"Init MIPI cameras driver",                        sample_init_mipi_camera  },
    {"Open camera",                                     sample_camera_open },
    {"Close camera",                                    sample_camera_close },
    {"sample_open_video_renderer",                      sample_open_video_renderer},
    {"sample_close_video_renderer",                     sample_close_video_renderer},
    {"Switch camera source on display",                 sample_switch_video_renderer_src},
    {"open nir led",                                    sample_open_nir_led  },
    {"close nir led",                                   sample_close_nir_led  },

    /*Power*/
    {"power mode : reset",                              sample_power_manager_reset },
    {"power mode : retention",                          sample_power_manager_sleep },
    {"power mode : deep retention",                     sample_power_manager_deep_sleep },
    {"power mode : RTC mode/Shutdown",                  sample_power_manager_shutdown },
    {"Quit",                                            sample_app_console_quit }
};

void console_thread(void *arg)
{
    unsigned int i = 0;
    unsigned int cmd_size = ARRAY_SIZE(console_cmd_array);
    char buf[256];
    console_cmd_func cmd_func;
    uint32_t rc = 1;

    while(1)
    {
        int id = 0;

        DSG("\n === Kdp520 sample app console Test Kit (%u) === ", cmd_size);
        for (i = 0; i < cmd_size; ++i)
        { 
            sprintf(buf, "(%2d) %s : ", i + 1, console_cmd_array[i].desc);
            DSG("%s", buf);
        }

        DSG(" command >> ");
        rc = kmdw_console_echo_gets(buf, sizeof(buf));
        if (!rc)
            continue;
        id = atoi(strtok(buf, " \r\n\t"));

        if (id > 0 && id < cmd_size)
        {
            cmd_func = console_cmd_array[id - 1].func;
            cmd_func();
        }
        else if (cmd_size == id) {
            cmd_func = console_cmd_array[id - 1].func;
            cmd_func();
            break;
        }
        else {
            err_msg("Invalid command: %d", id);
            continue;
        }
    }
}

void console_example_entry(void)
{
    tid_console_example = osThreadNew(console_thread, NULL, NULL);
    osThreadSetPriority (tid_console_example, osPriorityBelowNormal);
}

