#include "customizedCode.h"
#include "kmdw_memxfer.h"    //for flash access
#include "kmdw_memory.h"     //for ddr functions
#include "kmdw_console.h"
#include "kmdw_model.h"
#include "kdp_ddr_table.h"  //for default ddr table
#include <stdint.h>

extern const struct s_kdp_memxfer kdp_memxfer_module;

struct bounding_box_s_l {
    float x1;           // top-left corner:  x
    float y1;           // top-left corner:  y
    float x2;           // bottom-right corner:  x
    float y2;           // bottom-right corner:  y

    float score;        // probability score
    int class_num;      // class # (of many) with highest probability
};

#define YOLO_DETECTION_MAX_L    10

typedef struct yolo_result_s_l {
    uint32_t    class_count;            // total class count
    uint32_t    box_count;              /* boxes of all classes */
    struct bounding_box_s_l boxes[YOLO_DETECTION_MAX_L];       /* [box_count] */
} yolo_result_s_l_t;

void customer_model_run(void *argument) {

    // reserve DDR memory for image buffer
    const uint32_t img_size = 100352;
    uint32_t img_buf_addr = kmdw_ddr_reserve(img_size);

    // copy test image from flash to image buffer
    kdp_memxfer_module.init(MEMXFER_OPS_DMA, MEMXFER_OPS_DMA);
    kdp_memxfer_module.flash_to_ddr(img_buf_addr, 0xC66000/*flash addr*/, img_size);

    //reserve DDR memory for results
    struct yolo_result_s_l* p_out_p = NULL;
    p_out_p = (struct yolo_result_s_l*)kmdw_ddr_reserve(sizeof(struct yolo_result_s_l));

    //configure image
    //the test image is in shape of (224, 224, 3), RGB565 format
    int fmt = NPU_FORMAT_RGB565;
    struct kdp_img_cfg img_cfg = {
                .image_mem_addr         = img_buf_addr, // image address
                .image_mem_len          = 224*224*16,   // row * column * pixel_size
                .image_col              = 224,
                .image_row              = 224,
                .image_ch               = 3,
                .image_format           = fmt,          // image format, setting
                .image_buf_active_index = 0 };          // buffer index

    DSG("%d %d %d 0x%x ",img_cfg.image_col,
                         img_cfg.image_row,
                         img_cfg.image_ch,
                         img_cfg.image_format);

    DSG("[CUSTOMER] model type id [%d] \n", TINY_YOLO_V3);

    //run model
    kmdw_model_config_img(&img_cfg, NULL/*crop box*/, NULL/*padding value*/, NULL/*ext param*/);
    kmdw_model_config_result(0/*event*/, 0/*flag of event*/);
    int status = kmdw_model_run(
                           "model_yolo",   // tag
                           p_out_p,        // model result address
                           TINY_YOLO_V3,   // model type ID
                           false);         // is DME modent

    //print result
    struct yolo_result_s* p_fd_output_p =(struct yolo_result_s*) p_out_p;
    DSG("[CUSTOMER] OBJ_DET (classCount,BoxCount)= %d,%d\n",
        p_fd_output_p->class_count,
        p_fd_output_p->box_count);

    if (p_fd_output_p->box_count > 0) {
        int print_count = (p_fd_output_p->box_count > YOLO_DETECTION_MAX_L) ?
                                     YOLO_DETECTION_MAX_L : p_fd_output_p->box_count;
        DSG("(print %d boxes at most)\n", YOLO_DETECTION_MAX_L);

        for (int i = 0; i < print_count; i++) {
            DSG("box[%d]:class=%d, (x1,y1,x2,y2) = (%f, %f, %f, %f) @ score=%f\n", i,
                p_fd_output_p->boxes[i].class_num,
                p_fd_output_p->boxes[i].x1,
                p_fd_output_p->boxes[i].y1,
                p_fd_output_p->boxes[i].x2,
                p_fd_output_p->boxes[i].y2,
                p_fd_output_p->boxes[i].score);
        }
    }
}
