/*
 * KDP Inference support driver
 *
 * Copyright (C) 2019 Kneron, Inc. All rights reserved.
 *
 */
#include <string.h>
#include <stdlib.h>
#include "kmdw_display.h"
#include "framework/result.h"
#include "kapp.h"
#include "kapp_db.h"
#include "kapp_fid.h"
#include "kmdw_memory.h"
#include "kmdw_memxfer.h"
#include "kdrv_fb_mgr.h"
#include "kmdw_camera.h"
#include "media/kdp_inference.h"
#include "kmdw_console.h"
#include "ipc.h"
#include "power_manager.h"
#include "kdrv_pwm.h"
#include "kdrv_lcdc.h"
#include "kdp_ddr_table.h"
#include "kdp_flash_table.h"
#include "kl520_api.h"

int g_buf_idx[2] = {-1, -1};

#define NUM_REG_INFERENCE                   (20)
#define LM_SIZE                             (10)
#define LM_SIZE_BYTE                        (LM_SIZE * 4)
#define REG_TEMP_FR_SIZE                    (FM_SIZE_BYTE * NUM_REG_INFERENCE)
#define REG_TEMP_LM_SIZE                    (LM_SIZE_BYTE * NUM_REG_INFERENCE)

#define KDP_INF_WITHOUT_RESULT_ACK          (0)
#define KDP_INF_WITH_RESULT_ACK             (1)

static uint32_t g_snapshot_idx_addr[KDP_FLASH_SNAPSHOT_NUMS] ={
    KDP_FLASH_SNAPSHOT_IMG_00_ADDR,
    KDP_FLASH_SNAPSHOT_IMG_01_ADDR,
    KDP_FLASH_SNAPSHOT_IMG_02_ADDR,
    KDP_FLASH_SNAPSHOT_IMG_03_ADDR,
    KDP_FLASH_SNAPSHOT_IMG_04_ADDR,
    KDP_FLASH_SNAPSHOT_IMG_05_ADDR,
    KDP_FLASH_SNAPSHOT_IMG_06_ADDR,
    KDP_FLASH_SNAPSHOT_IMG_07_ADDR,
    KDP_FLASH_SNAPSHOT_IMG_08_ADDR,
    KDP_FLASH_SNAPSHOT_IMG_09_ADDR,
};

static kapp_fid_inout_t         *m_fdr_inout = NULL;

static struct facedet_result_s  *m_fd_out  = NULL;
static struct landmark_result_s *m_lm_out  = NULL;
static struct fr_result_s       *m_fr_out  = NULL;

static struct facedet_result_s  *m_fd_1_out  = NULL;
static struct landmark_result_s *m_lm_1_out  = NULL;
static struct fr_result_s       *m_fr_1_out  = NULL;

static kapp_db_user_data_t      *m_db_data = NULL;
static uint32_t *db_tmp_lm = 0;
static float    *db_tmp_fr = 0;
static uint32_t *db_tmp_1_lm = 0;
static float    *db_tmp_1_fr = 0;

static bool db_tmp_valid_pose[NUM_REG_INFERENCE];
static bool g_fdfr_run_inference = FALSE;
static bool g_en_snapshot_fdfr[2] = {FALSE, FALSE};

static int g_snapshot_idx = -1;
static fdr_result g_fdr_ret;
static uint32_t g_draw_timeout_ms = osWaitForever;

/* For fdfr inference result */
struct kdp_rect m_boundingbox = { 0 };
struct kdp_rect s_boundingbox = { 0 };
/* For face hint boundingbox */
struct kdp_rect h_boundingbox = { 0 };

enum kdp_inference_imgsrc m_inf_imgsrc = KDP_INFERENCE_SINGLE_IMGSRC;


osThreadId_t tid_inference = NULL;
osThreadId_t tid_draw = NULL;

int g_draw_boundingbox_mode = ALL_DRAW_BOUNDINGBOX_MODE;
int m_face_add_timeout = 0;
int m_face_recognition_timeout = 0;
u32 m_fdfr_tick = 0;

int reg_type = 0;
int reg_count = 0;


static kdp_fsm g_fsm;
static int g_max_fsm_num = 0;
static kdp_fsm_table g_fsm_table[] = 
{
    {KDP_FDR_FSM_INFERENCE, FLAGS_KDP520_START_INFERENCE_EVT, kdp_fdr_do_inference,  KDP_FDR_FSM_IDLE},
    {KDP_FDR_FSM_INFERENCE, FLAGS_KDP520_START_INFERENCE_LOOP_EVT, kdp_fdr_do_inference, KDP_FDR_FSM_IDLE},
    {KDP_FDR_FSM_INFERENCE, FLAGS_KDP520_STOP_INFERENCE_LOOP_EVT, kdp_fdr_do_inference,  KDP_FDR_FSM_IDLE},
    {KDP_FDR_FSM_REGISTER, FLAGS_KDP520_REGISTER_FDR_EVT, kdp_fdr_do_register, KDP_FDR_FSM_IDLE},
    {KDP_FDR_FSM_REGISTER, FLAGS_KDP520_AUTO_REGISTER_FDR_EVT, kdp_fdr_do_register, KDP_FDR_FSM_IDLE},
    {KDP_FDR_FSM_REGISTER, FLAGS_KDP520_REGISTER_CFM_FDR_EVT, kdp_fdr_do_register, KDP_FDR_FSM_IDLE},
    {KDP_FDR_FSM_REGISTER, FLAGS_KDP520_AUTO_REGISTER_BOTH_FDR_EVT, kdp_fdr_do_register, KDP_FDR_FSM_IDLE},
    {KDP_FDR_FSM_REGISTER, FLAGS_KDP520_REGISTER_CFM_BOTH_FDR_EVT, kdp_fdr_do_register, KDP_FDR_FSM_IDLE},
    {KDP_FDR_FSM_REGISTER, FLAGS_KDP520_REGISTER_FDR_EVT, kdp_fdr_do_register, KDP_FDR_FSM_IDLE},
    {KDP_FDR_FSM_CANCEL_REGISTER, FLAGS_KDP520_CANCEL_REGISTER_FDR_EVT, kdp_fdr_do_cancel_register, KDP_FDR_FSM_IDLE},
    {KDP_FDR_FSM_ADD, FLAGS_KDP520_ADD_FDR_FACE_EVT, kdp_fdr_do_add, KDP_FDR_FSM_IDLE},
    {KDP_FDR_FSM_DELETE, FLAGS_KDP520_DEL_FDR_FACE_EVT, kdp_fdr_do_delete, KDP_FDR_FSM_IDLE},
};

char *g_kdp_fdr_fsm_str[] = {
    "KDP_FDR_FSM_UNINIT",
    "KDP_FDR_FSM_IDLE",
    "KDP_FDR_FSM_INFERENCE",
    "KDP_FDR_FSM_REGISTER",
    "KDP_FDR_FSM_CANCEL_REGISTER",
    "KDP_FDR_FSM_ADD",
    "KDP_FDR_FSM_DELETE"
};
    
kdp_all_comm_params m_kdp_all_comm_params;

kdp_camera_ctx m_kdp_cam_ctx[IMGSRC_NUM + 2] = {
    {FALSE, CAMERA_STATE_IDLE, 0, 0, 0},
    {FALSE, CAMERA_STATE_IDLE, 0, 0, 0},
};


/* Middleware layer */
/*-----------------------------------------------------------------------------------------
 *                                  Private power manager API
 *-----------------------------------------------------------------------------------------*/
int ncpu_inference_nap(enum pm_device_id dev_id)
{
    if(g_fdfr_run_inference == TRUE)
        return -1;
    else
        return 0;
}

int ncpu_inference_deep_nap(enum pm_device_id dev_id)
{
    if(g_fdfr_run_inference == TRUE)
        return -1;
    else
        return 0;
}

int ncpu_inference_sleep(enum pm_device_id dev_id)
{
    if(g_fdfr_run_inference == TRUE)
        return -1;
    else
        return 0;
}

int ncpu_inference_deep_sleep(enum pm_device_id dev_id)
{
    if(g_fdfr_run_inference == TRUE)
        return -1;
    else
        return 0;
}

struct pm_s pm_ncpu_inference =
{
    .nap = ncpu_inference_nap,
    .wakeup_nap = 0,
    .deep_nap = ncpu_inference_deep_nap,
    .wakeup_deep_nap = 0,
    .sleep = 0,
    .wakeup_sleep = ncpu_inference_sleep,
    .deep_sleep = 0,
    .wakeup_deep_sleep = ncpu_inference_deep_sleep,
};

/*-----------------------------------------------------------------------------------------
 *                                  Private kdp draw API
 *-----------------------------------------------------------------------------------------*/
#ifdef BOARD_96
static void _kdp_draw_fdfr_result(fdr_result fdr_result)
{
    u32 buf_addr;
    int buf_idx;

    if(kdrv_display_get_review_snapshot_en() == TRUE) {
        buf_addr = kmdw_video_renderer_get_buffer_addr();
    }
    else {
        buf_addr = kdrv_fb_mgr_next_read(CAMERA_RGB_IDX, &buf_idx);
    }
    kmdw_display_update_draw_fb(buf_addr, CAMERA_RGB_IDX);

    if (fdr_result >= FDR_RET_OK) {
        if (fdr_result == FDR_RET_OK) {
            kmdw_display_set_pen_rgb565(YELLOW, 4);
        } else if (fdr_result == FDR_RET_DB_COMP_OK) {
            kmdw_display_set_pen_rgb565(GREEN, 4);
        } else {
            kmdw_display_set_pen_rgb565(RED, 4);
        }

        /*Avoid drawing the boundingbox on this frame buffer which is under the FDFR inference procedure*/
        if(buf_addr != kapp_get_img_mem_addr(CAMERA_RGB_IDX))
        {
        if(fdr_result != FDR_RET_NO_FACE) {
            if(g_draw_boundingbox_mode & DRAW_FDR_RESULT_BOUNDINGBOX) {
                kmdw_display_draw_rect(m_boundingbox.start_x, m_boundingbox.start_y, m_boundingbox.end_x - m_boundingbox.start_x, m_boundingbox.end_y - m_boundingbox.start_y, DRAW_FDR_RESULT_BOUNDINGBOX);
            }
        }
        }
        if(g_draw_boundingbox_mode & DRAW_HINT_BOUNDINGBOX)  {
            if(g_draw_boundingbox_mode == DRAW_HINT_BOUNDINGBOX)
            {
                if (fdr_result == FDR_RET_OK) {
                    kmdw_display_set_pen_rgb565(YELLOW, 8);
                } else if (fdr_result == FDR_RET_DB_COMP_OK) {
                    kmdw_display_set_pen_rgb565(GREEN, 8);
                } else {
                    kmdw_display_set_pen_rgb565(RED, 8);
                }
            }
            else if(g_draw_boundingbox_mode == ALL_DRAW_BOUNDINGBOX_MODE)
                kmdw_display_set_pen_rgb565(BLUE, 8);

            kmdw_display_draw_rect(s_boundingbox.start_x, s_boundingbox.start_y, s_boundingbox.end_x - s_boundingbox.start_x, s_boundingbox.end_y - s_boundingbox.start_y, DRAW_HINT_BOUNDINGBOX);
        }
    }

    if(kdrv_display_get_review_snapshot_en() == FALSE)
        kdrv_fb_mgr_read_done(CAMERA_RGB_IDX, buf_idx);
}
#endif

/*-----------------------------------------------------------------------------------------
 *                                  Private kdp camera API
 *-----------------------------------------------------------------------------------------*/
static int _kdp_camera_open(uint8_t cam_idx)
{
    int ret;

    struct cam_capability cap;
    struct cam_format fmt;

    memset(&cap, 0, sizeof(cap));
    memset(&fmt, 0, sizeof(fmt));

    if (0 != (ret = kmdw_camera_open(cam_idx)))
        return ret;

    if (0 != (ret = kmdw_camera_get_device_info(cam_idx, &cap)))
        return ret;

    fmt.width = m_kdp_cam_ctx[cam_idx].width;
    fmt.height = m_kdp_cam_ctx[cam_idx].height;
    fmt.pixelformat = m_kdp_cam_ctx[cam_idx].pixelformat;
    //dbg_msg("[%s] cam_idx:%d, width:%d, height:%d,  pixelformat: %d", __func__, cam_idx, fmt.width, fmt.height, fmt.pixelformat);
    if (0 != (ret = kmdw_camera_set_frame_format(cam_idx, (struct cam_format *)&fmt)))
        return ret;

    if (0 != (ret = kmdw_camera_get_frame_format(cam_idx, &fmt)))
        return ret;

    if (0 != (ret = kmdw_camera_buffer_init(cam_idx)))
        return ret;

    m_kdp_cam_ctx[cam_idx].state = CAMERA_STATE_INITED;
    return 0;
}

static int _kdp_camera_start(unsigned int cam_idx)
{
    int ret = -1;

    if (0 != (ret = kmdw_camera_start(cam_idx))) {
        return ret;
    }

    m_kdp_cam_ctx[cam_idx].state = CAMERA_STATE_RUNNING;
    return 0;
}

static int _kdp_camera_stop(unsigned int cam_idx)
{
        int ret = -1;
    if(0 != (ret = kmdw_camera_stop(cam_idx)))
        return ret;

    m_kdp_cam_ctx[cam_idx].state = CAMERA_STATE_INITED;
    return 0;
}

static int _kdp_camera_close(unsigned int cam_idx)
{
    int ret = -1;

    if(0 != (ret = kmdw_camera_close(cam_idx)))
        return ret;

    m_kdp_cam_ctx[cam_idx].state = CAMERA_STATE_IDLE;
    return ret;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
 *                                          Private kdp FDFR API
 * From sample_app_fdr.c
 *
 * _sample_kneron_reg_fdr_only(struct kdp_rect *lp_rect, int index)     (Combine together, use parameter "num_rgst" to distinguish them)
 * _sample_kneron_reg_fdr_both_only(struct kdp_rect *lp_rect, int index) -> _kdp_rgst_fdfr_only(struct kdp_rect *lp_rect, int index, int num_rgst)
 * sample_kneron_fdr_registration(u32 curr_user_idx)                     -> _kdp_fdfr_registration(uint32_t user_idx)
 * sample_kneron_fdr_auto_reg(u32 curr_user_idx, int reg_index)         (Combine together, use parameter "num_rgst" to distinguish them)
 * sample_kneron_fdr_both_reg(u32 curr_user_idx, int reg_index)          -> _kdp_fdfr_auto_registeration(uint32_t user_idx, int reg_index, int num_reg)
 * sample_kneron_fdr_registration_confirm(u32 curr_user_idx)            (Combine together, use parameter "num_rgst" to distinguish them)
 * sample_kneron_fdr_both_registration_confirm(u32 curr_user_idx)        ->_kdp_fdfr_registration_confirm(uint32_t user_idx, int num_rgst);
 *
 * sample_kneron_fdr_add_user(u32 curr_user_idx)                    ->  _kdp_fdfr_add_user(uint32_t user_idx)
 * sample_kneron_fdr_del_user(unsigned char all, u32 curr_user_idx) ->  _kdp_fdfr_del_user(bool all, uint32_t user_idx)
 * sample_kneron_fdr_cancel(void)                                   ->  _kdp_fdfr_cancel_registeration(void)
 * sample_kneron_fdr_abort(void)                                    ->  _kdp_fdfr_abort(void)
 *-----------------------------------------------------------------------------------------------------------------------------------*/
#define INIT_TILE2
#define Max_EXPO_TIME 43072
uint32_t nir_frm_cnt = 0;
static int rgb_fd_get_flag = 10;
static u32 m_iCurExpoTime = 43072;
//u8 tile_val[20];
//static int init_tile = 0;
//static u8 led_flag = 0;
//static u8 initTile_flag = 0;
//static u32 recognition_cnt = 0;
//static u16 preprocess_cnt = 0;
//static u8 distance_count = 0;
//static u32 rgb_light_cnt = 0;
//static u8 RgbLedFlag = 0;
//static u8 CurLedPwm = 0;
//static u8 TgtLedPwm = 99;
//static u8 nir_mode = 0; // 0 for room,  1 for light
//static float bias_scale= 1;
static struct cam_sensor_aec rgb_aec = {
    20, 80, 0, 60,
    0, 0, 0, 0,
};

int _kdp_check_DB_space(uint32_t user_idx)
{
    if ((user_idx + 1) > DB_SLOT_SIZE)/*User id(curr_user_idx) starts from 0*/
    {
        info_msg("[registration] : No space\n");
        return KAPP_UNKNOWN_ERR;
    }
    return 0;
}


#include "kdev_wifi.h"
extern uint8_t kapp_tcp_status;
extern socket_info kapp_socket;

fdr_result _kdp_fdfr_single_inference(void)
{
    fdr_result fdr_ret = FDR_RET_OOPS;
    int ret = -1;
    struct kdp_rect *lp_rect = &m_boundingbox;
    float score;
	
		//char fdr_result_res[] = "face detected successful: \n";
		char fdr_result_res[100] = {0};
		//char lm_result_res[70] = "lm pos:";
		//char fm_res[10] = "fm info:\n";

    m_fdr_inout->op = KAPP_FID_OP_INFERENCE;
    m_fdr_inout->thresh_fid[KAPP_IMG_SRC_RGB] = 0.0f; //use default threshold value

    do {
        //todo dummy
        memset(m_fd_out, 0, sizeof(struct facedet_result_s));
        memset(m_fr_out, 0, sizeof(struct fr_result_s));

        // get exp time
        u16 a = 0;
        u8 b, c, d, e;
        //todo need rename
        kmdw_camera_get_lux(CAMERA_RGB_IDX, &a, &b, &c, &d, &e);
        ////dbg_msg("exp_rgb_time = %d", a);

        ret = kapp_fdr(KAPP_IMG_SRC_RGB, m_fdr_inout, 
			                 m_fd_out, m_lm_out, m_fr_out, &score);
        if (ret == KAPP_FDR_NO_FACE ){
            info_msg("[inference %s %d] FD result : NO FACE\n", __func__, __LINE__);

            // if no face, aec_roi set as 480x480 rgb right img
            if (rgb_fd_get_flag == 0){
                ////dbg_msg(" rgb_fd_get_flag  = 0 ");
            }else{
                //todo need rename
                rgb_fd_get_flag = 0;
                rgb_aec.center_x1 = 30;
                rgb_aec.center_x2 = 70;
                rgb_aec.center_y1 = 10;
                rgb_aec.center_y2 = 50;
                kmdw_camera_set_aec(CAMERA_RGB_IDX, &rgb_aec);
                ////dbg_msg("set rgb_fd_get_flag = 0 ");
            }
        } else if (ret == KAPP_FDR_WRONG_USAGE || ret == KAPP_FDR_BAD_POSE ||
                   ret == KAPP_FDR_FR_FAIL || ret == KAPP_LM_LOW_CONFIDENT || ret == KAPP_LM_BLUR) {
            info_msg("other than KAPP_FDR_NO_FACE ret : %d\n", ret);
        }else{ // if get face, aec_roi set as fd region.
					
						if(kapp_tcp_status)
						{
							//kdev_socket_send(kapp_socket.sock,fdr_result_res,strlen(fdr_result_res));
							memset(fdr_result_res,0,sizeof(fdr_result_res));
							sprintf(fdr_result_res,"fd pos: X:%d,Y:%d,W:%d,H:%d\nlm pos: [%d,%d],[%d,%d],[%d,%d],[%d,%d],[%d,%d]\nFM info:\n",
							m_fd_out->xywh[0],
							m_fd_out->xywh[1],
							m_fd_out->xywh[2],
							m_fd_out->xywh[3],
							m_lm_out->marks[0].x,m_lm_out->marks[0].y,
							m_lm_out->marks[1].x,m_lm_out->marks[1].y ,
							m_lm_out->marks[2].x,m_lm_out->marks[2].y ,
							m_lm_out->marks[3].x,m_lm_out->marks[3].y ,
							m_lm_out->marks[4].x,m_lm_out->marks[4].y
							);
							kdev_socket_send(kapp_socket.sock,fdr_result_res,strlen(fdr_result_res));
							/*
							memset(fdr_result_res,0,sizeof(fdr_result_res));
							sprintf(fdr_result_res,"lm pos: [%d,%d],[%d,%d],[%d,%d],[%d,%d],[%d,%d]\n FM info:\n",
									m_lm_out->marks[0].x,m_lm_out->marks[0].y,
									m_lm_out->marks[1].x,m_lm_out->marks[1].y,
									m_lm_out->marks[2].x,m_lm_out->marks[2].y,
									m_lm_out->marks[3].x,m_lm_out->marks[3].y,
									m_lm_out->marks[4].x,m_lm_out->marks[4].y);
							kdev_socket_send(kapp_socket.sock,fdr_result_res,strlen(fdr_result_res));*/
							
							kdev_socket_send(kapp_socket.sock,m_fr_out,sizeof(struct fr_result_s));
							
						}
            lp_rect->start_x = (u16) m_fd_out->xywh[0];
            lp_rect->start_y = (u16) m_fd_out->xywh[1];
            lp_rect->end_x = ((u16) m_fd_out->xywh[0]) + ((u16) m_fd_out->xywh[2]);
            lp_rect->end_y = ((u16) m_fd_out->xywh[1]) + ((u16) m_fd_out->xywh[3]);

            u8 xmin_inter = (u8) (m_fd_out->xywh[0] >> 3);
            u8 xmax_inter = (u8) ( (m_fd_out->xywh[0] + m_fd_out->xywh[2]) >> 3 );
            u8 ymin_inter = (u8) (m_fd_out->xywh[1] >> 3);
            u8 ymax_inter = (u8) ( (m_fd_out->xywh[1] + m_fd_out->xywh[3]) >> 3 );

            if(xmin_inter >= 20){ // if get face and fd region in rgb right region(480x480)
                if (rgb_fd_get_flag == 1){
                    ////dbg_msg(" rgb_fd_get_flag = 1 ");
                }else{
                    rgb_fd_get_flag = 1;
                    rgb_aec.center_x1 = xmin_inter;
                    rgb_aec.center_x2 = xmax_inter;
                    rgb_aec.center_y1 = ymin_inter;
                    rgb_aec.center_y2 = ymax_inter;
                    kmdw_camera_set_aec(CAMERA_RGB_IDX, &rgb_aec);
                }
            }else{ // if fd region not in rgb right region (480x480)
                if (rgb_fd_get_flag == 0){
                    ////dbg_msg(" rgb_fd_get_flag  = 0 ");
                }else{
                    rgb_fd_get_flag = 0;
                    rgb_aec.center_x1 = 30;
                    rgb_aec.center_x2 = 70;
                    rgb_aec.center_y1 = 10;
                    rgb_aec.center_y2 = 50;
                    kmdw_camera_set_aec(CAMERA_RGB_IDX, &rgb_aec);
                    ////dbg_msg("set rgb_fd_get_flag = 0 ");
                }
            }

        }

        if (KAPP_OK == ret){
            fdr_ret = FDR_RET_DB_COMP_OK;
        }else{
            fdr_ret = FDR_RET_DB_COMP_FAIL;
        }

    } while (0);

    if (ret == KAPP_FDR_NO_FACE) {
        fdr_ret = FDR_RET_NO_FACE;
    }

    return fdr_ret;
}

static int _kdp_rgst_fdfr_only(struct kdp_rect *lp_rect, int index, int num_rgst)
{
    int ret, ret0 = 0, ret1 = 0;
    kapp_fid_inout_t nir_inout = { .op = KAPP_FID_OP_EXTRACT };
    float score;

    do {
        m_fdr_inout->op = KAPP_FID_OP_EXTRACT;
        if(1 == num_rgst) {
            ret = kapp_fdr(KAPP_IMG_SRC_RGB, m_fdr_inout, m_fd_out,
                              (struct landmark_result_s*)&db_tmp_lm[index*10],
                              (struct fr_result_s*)&db_tmp_fr[index*512],
															&score);
            //dbg_msg("[%s] ret=0x%x\n", __func__, ret);

            if (KAPP_FDR_NO_FACE != ret)
            {
                lp_rect->start_x = (u16) m_fd_out->xywh[0];
                lp_rect->start_y = (u16) m_fd_out->xywh[1];
                lp_rect->end_x = ((u16) m_fd_out->xywh[0]) + ((u16) m_fd_out->xywh[2]);
                lp_rect->end_y = ((u16) m_fd_out->xywh[1]) + ((u16) m_fd_out->xywh[3]);
            }
            else {
                info_msg("[inference %s %d] FD result : NO FACE\n", __func__, __LINE__);
                break;
            }

        }
        else if(2 == num_rgst) {
            ret0 = kapp_fdr(KAPP_IMG_SRC_RGB, m_fdr_inout, m_fd_out,
                               (struct landmark_result_s*)&db_tmp_lm[index*10],
                               (struct fr_result_s*)&db_tmp_fr[index*512],
																&score);
            ret1 = kapp_fdr(KAPP_IMG_SRC_NIR, &nir_inout, m_fd_1_out,
                               (struct landmark_result_s*)&db_tmp_1_lm[index*10],
                               (struct fr_result_s*)&db_tmp_1_fr[index*512],
																&score);
            //dbg_msg("[%s] ret0=0x%x, ret1=0x%x\n", __func__, ret0, ret1);

            if ((KAPP_OK == ret0) && (KAPP_OK == ret1))
            {
                lp_rect->start_x = (u16) m_fd_out->xywh[0];
                lp_rect->start_y = (u16) m_fd_out->xywh[1];
                lp_rect->end_x = ((u16) m_fd_out->xywh[0]) + ((u16) m_fd_out->xywh[2]);
                lp_rect->end_y = ((u16) m_fd_out->xywh[1]) + ((u16) m_fd_out->xywh[3]);
                ret = ret0; //might be some cases left for different status of ret1
            }
            else {
                ret = KAPP_FDR_NO_FACE;
                info_msg("[inference %s %d] FD result : NO FACE\n", __func__, __LINE__);
                break;
            }
        }
    } while (0);

    return ret;
}

int _kdp_fdfr_registration(uint32_t user_idx)
{
    int ret = -1;
    float score;

    do {
        //todo : should restructure
        if(0 != (ret = _kdp_check_DB_space(user_idx))) {
            break;
        }
        else if (kapp_db_get_fm0_cnt(user_idx) >= MAX_FID) {
            info_msg("[registration] : saved_fmap_cnt > MAX_FID(5)\n");
            ret = KAPP_OK;
            break;
        }
        else {
            memset(m_fd_out, 0, sizeof(struct facedet_result_s));
            memset(m_fr_out, 0, sizeof(struct fr_result_s));

            m_fdr_inout->op = KAPP_FID_OP_REGISTER;
            int fm_available = 1;
            int curr_fm = kapp_db_get_fm0((int)user_idx);
            while (1) {
                if ((curr_fm & (1 << (fm_available - 1))) == 0) break;
                fm_available++;
            }
            m_fdr_inout->face_id = fm_available;  // insert to fm not occupied
            m_fdr_inout->user_id = (uint16_t)user_idx;
            m_fdr_inout->thresh_fid[KAPP_IMG_SRC_RGB] = 0.0f; //use default threshold value

            ret = kapp_fdr(KAPP_IMG_SRC_RGB, m_fdr_inout, 
						               m_fd_out, m_lm_out, m_fr_out, &score);
            if (KAPP_OK == ret) {
                info_msg("[registration] : -- OK!\n\n");
            }
            else if (KAPP_DB_ALREADY_SAVED == ret) {
                info_msg("[registration] : -- ERROR!, face has been registered\n\n");
            }
            else {
                info_msg("[registration] : -- ERROR!, err=0x%x\n\n", ret);
            }
            break;
        }
    } while (0);

    return ret;
}

static int _kdp_fdfr_auto_registeration(uint32_t user_idx, int reg_index, int num_rgst)
{
    //dbg_msg("[%s %d] user_idx:%d, reg_index:%d, num_rgst:%d", __func__, __LINE__, user_idx, reg_index, num_rgst);
    int ret = -1;
    db_tmp_valid_pose[reg_index] = false;

    if(0 != (ret = _kdp_check_DB_space(user_idx))) {
        return ret;
    }

    // FIXME: _sample_kneron_fdr_both_only() is going to be deleted, how to remove it in the following comment block?
    /*_kdp_rgst_fdr_only() combines _sample_kneron_fdr_both_only() + _sample_kneron_reg_fdr_both_only()
      num_reg = 1 => _sample_kneron_fdr_both_only()
      num_reg = 2 => _sample_kneron_reg_fdr_both_only()*/
    ret = _kdp_rgst_fdfr_only(&m_boundingbox, reg_index, num_rgst);

    //for (int j=0; j<10; j++) {
    //    DSG_NOLF(" lm %d is %d!\n", j, db_tmp_lm[reg_index*10+j]);
    //}

    if (ret == KAPP_OK) {
        //DSG_NOLF(" have face!\n");
        //kmdw_printf("start_x: %hu\n", m_boundingbox.start_x);
        //kmdw_printf("start_y: %hu\n", m_boundingbox.start_y);
        //kmdw_printf("end_x: %hu\n", m_boundingbox.end_x);
        //kmdw_printf("end_y: %hu\n", m_boundingbox.end_y);
        //check if it is bad pose
        if(1 == num_rgst) {
            bool bad_pose = kapp_check_bad_pose(&db_tmp_lm[reg_index*10]);
            if (!bad_pose) {
                //DSG_NOLF(" good pose for index %d!\n", reg_index);
                db_tmp_valid_pose[reg_index] = true;
            }
        }
        else if(2 == num_rgst) {
            //check if it is bad pose
            bool rgb_bad_pose = kapp_check_bad_pose(&db_tmp_lm[reg_index*10]);
            bool nir_bad_pose = kapp_check_bad_pose(&db_tmp_1_lm[reg_index*10]);
            if (!rgb_bad_pose && !nir_bad_pose) {
                //DSG_NOLF(" good pose for index %d!\n", reg_index);
                db_tmp_valid_pose[reg_index] = true;
            }
        }
    }

    return ret;
}


static int _kdp_fdfr_registration_confirm(uint32_t user_idx, int num_rgst) {
    info_msg(" Registration confrim for id %d!\n", user_idx);
    int ret = -1;
    int near_cnt = 0;
    float score;

    m_fdr_inout->thresh_fid[KAPP_IMG_SRC_RGB] = 0.0f; //use default threshold value

    //need to check if the user is already in db??
    for (int i = 0; i<NUM_REG_INFERENCE; i++) {
        //DSG_NOLF("check %d FM!\n", i);
        if (db_tmp_valid_pose[i]) {
            //DSG_NOLF("valid pose %d!\n", i);
            ret = kapp_db_compare((uint32_t)&db_tmp_fr[i*512], &m_fdr_inout->user_id, &score,
                        					m_fdr_inout->thresh_fid[KAPP_IMG_SRC_RGB]);
            if(ret != KAPP_DB_NO_MATCH) {
                //DSG_NOLF("%d FM match with %d in database!\n", i, m_fdr_inout->user_id);
                near_cnt++;
                db_tmp_valid_pose[i] = 0;
                //return KAPP_DB_ALREADY_SAVED; // person already in DB ??
            }
        }
        //KAPP_DB_NO_MATCH
    }
        if (near_cnt>5) {
            //dbg_msg("more than 5 FM are too close to those in database!\n");
            return KAPP_DB_ALREADY_SAVED; // person already in DB ??
        }
    //check best pose for registration

    int index[5], index1[5], index2[5], index3[5], index4[5];

    float tmp_lm[5];
    float min_lm[5];
    float min_lm1[5], min_lm2[5], min_lm3[5], min_lm4[5];
    //initialize
    for (int i=0; i<5; i++) {
        min_lm[i]  = 1000;
        min_lm1[i] = 1000;
        min_lm2[i] = 1000;
        min_lm3[i] = 1000;
        min_lm4[i] = 1000;
        index[i]  = 0;
        index1[i] = 0;
        index2[i] = 0;
        index3[i] = 0;
        index4[i] = 0;
    }
    info_msg("check for best pose!\n");
    for (int i=0;i<NUM_REG_INFERENCE;i++) {
        if (db_tmp_valid_pose[i]) {
            //DSG_NOLF("pose number %d!\n",i);
            kapp_get_pose_value(&db_tmp_lm[i*10], tmp_lm);
            for (int j=0; j<5; j++) {
                bool change = FALSE;
                if (tmp_lm[j] < min_lm[j]) {
                    index[j] = i;
                    min_lm[j] = tmp_lm[j];
                    change = TRUE;
                }
                if (!change && (tmp_lm[j] < min_lm1[j])) {
                        index1[j] = i;
                        min_lm1[j] = tmp_lm[j];
                        change = TRUE;
                }
                if (!change && (tmp_lm[j] < min_lm2[j])) {
                    index2[j] = i;
                        min_lm2[j] = tmp_lm[j];
                        change = TRUE;
                }
                if (!change && (tmp_lm[j] < min_lm3[j])) {
                        index3[j] = i;
                        min_lm3[j] = tmp_lm[j];
                        change = TRUE;
                }
                if (!change && (tmp_lm[j] < min_lm4[j])) {
                        index4[j] = i;
                        min_lm4[j] = tmp_lm[j];
                        change = TRUE;
                }
            }
        }
    }
    //check if index has repeated
    //DSG_NOLF("check if any index is repeat!\n");
    int target;
    for (int i= 0; i<5; i++) {
        target = index[i];
        bool repeat = FALSE;
        for (int j=0; j<5; j++) {
            if ((i!=j)&&(target == index[j])) {
                repeat = TRUE;
                break;
            }
        }
        if (!repeat) {
            //DSG_NOLF("keep index %d for pose %d!\n", target, i);
            index[i] = target;
            continue;
        }
        target = index1[i];
        repeat = FALSE;
        for (int j=0; j<5; j++) {
            if ((i!=j)&&(target == index[j])) {
                repeat = TRUE;
                break;
            }
        }
        if (!repeat) {
            //DSG_NOLF("replace min 1 index %d for pose %d!\n", target, i);
            index[i] = target;
            continue;
        }
        target = index2[i];
        repeat = FALSE;
        for (int j=0; j<5; j++) {
            if ((i!=j)&&(target == index[j])) {
                    repeat = TRUE;
                    break;
            }
        }
        if (!repeat) {
            //DSG_NOLF("replace min 2 index %d for pose %d!\n", target, i);
            index[i] = target;
            continue;
        }
        target = index3[i];
        repeat = FALSE;
        for (int j=0; j<5; j++) {
            if ((i!=j)&&(target == index[j])) {
                    repeat = TRUE;
                    break;
            }
        }
        if (!repeat) {
            //DSG_NOLF("replace min 3 index %d for pose %d!\n", target, i);
            index[i] = target;
            continue;
        }
        target = index4[i];
        repeat = FALSE;
        for (int j=0; j<5; j++) {
            if ((i!=j)&&(target == index[j])) {
                    repeat = TRUE;
                    break;
            }
        }
        if (!repeat) {
            //DSG_NOLF("replace min 4 index %d for pose %d!\n", target, i);
            index[i] = target;
            continue;
        }
    }
    info_msg("best pose index %d %d %d %d %d!\n", index[0], index[1], index[2], index[3], index[4]);
    //check if these lm valid
    for (int i=0; i<5; i++) {
        if (!db_tmp_valid_pose[index[i]]) {
            critical_msg("ERROR! not enough valid pose in registration!");
            ret = KAPP_FDR_BAD_POSE;
            return ret;
        }
    }
    //DSG_NOLF("register for user id %d!\n", curr_user_idx);
    if(1 == num_rgst) {
        for (int i=0; i<5; i++) {
            uint32_t fr_addr = (uint32_t)db_tmp_fr+index[i]*512*4;
            //DSG_NOLF("db_tmp_fr addr is %d, index[i]*512 is %d, fr_addr is %d!\n", db_tmp_fr, index[i]*512, fr_addr);
            ret = kapp_db_register(fr_addr, user_idx, i+1);
        }
    }
    else if(2 == num_rgst) {
        for (int i=0; i<5; i++) {
            uint32_t rgb_fr_addr = (uint32_t)db_tmp_fr+index[i]*512*4;
            uint32_t nir_fr_addr = (uint32_t)db_tmp_1_fr+index[i]*512*4;
            //DSG_NOLF("db_tmp_fr addr is %d, index[i]*512 is %d, fr_addr is %d!\n", db_tmp_fr, index[i]*512, fr_addr);
            ret = kapp_db_both_register(rgb_fr_addr, nir_fr_addr, user_idx, i+1);
        }
    }
    return ret;
}

static int _kdp_fdfr_add_user(uint32_t user_idx)
{
    int ret = -1;

    do {
        //todo no need
        if(0 != (ret = _kdp_check_DB_space(user_idx))) {
            break;
        }
        else {
            m_db_data->user_id_in = (uint16_t)user_idx;
            ret = kapp_db_add(m_db_data);
        }

        if (KAPP_OK == ret) {
            info_msg("[add_user] : UID:%d, OK!\n", user_idx);
        }
        else {
            info_msg("[add_user] : UID:%d, ERROR!, ret=%x\n", user_idx, ret);
            break;
        }
    } while (0);
    return ret;
}

static int _kdp_fdfr_del_user(bool all, uint32_t user_idx)//int user_id)
{
    int ret = -1;
    do {
        if (all)
            m_db_data->del_all = TRUE;
        else {
            m_db_data->del_all = FALSE;
            m_db_data->user_id_in = (uint16_t)user_idx;
        }
        ret = kapp_db_delete(m_db_data);
        if (KAPP_OK == ret) {
            info_msg("[delete user face] delete user face -- OK!\n");
        }
        else {
            info_msg("[delete user face] delete user face -- ERROR, ret=%x\n", ret);
            break;
        }
        ret = 0;
    } while (0);

    return ret;
}

static int _kdp_fdfr_cancel_registeration(void)
{
    int ret = -1;

    /* under construction */
    do {
         ret = kapp_db_abort_reg();
         if (KAPP_OK == ret) {
             info_msg("[cancel registration] -- OK!\n");
         } else {
             info_msg("[cancel registration] -- ERROR!, err=%x\n", ret);
         }
         ret = 0;
    } while (0);

    //caller should release the memory which alloc in sample_kneron_fdr_model_init();
    return ret;
}

static unsigned int _kdp_fdfr_fb_capture(void)
{
    int cam_idx;
    unsigned int frame_addr_rgb, frame_addr_nir;

    cam_idx = CAMERA_RGB_IDX;
    frame_addr_rgb = kdrv_fb_mgr_next_inf(cam_idx, &g_buf_idx[cam_idx]);
    if(kdrv_display_get_review_snapshot_en() == TRUE) {
        frame_addr_rgb = KDP_DDR_SNAPSHOT_RGB_IMG_ADDR;
    }
    kapp_set_img_mem_addr(cam_idx, frame_addr_rgb);
    kdp_fdfr_snapshot(cam_idx, frame_addr_rgb);
    cam_idx = CAMERA_NIR_IDX;
    frame_addr_nir = kdrv_fb_mgr_next_inf(cam_idx, &g_buf_idx[cam_idx]);
    if(kdrv_display_get_review_snapshot_en() == TRUE) {
        frame_addr_nir = KDP_DDR_SNAPSHOT_NIR_IMG_ADDR;
    }
    kapp_set_img_mem_addr(cam_idx, frame_addr_nir);
    kdp_fdfr_snapshot(cam_idx, frame_addr_nir);
    return frame_addr_rgb;
}

static void _kdp_frdr_fb_prepare(void)
{
    int cam_idx;

    cam_idx = CAMERA_RGB_IDX;
    kdrv_fb_mgr_inf_done(cam_idx, g_buf_idx[cam_idx]);
    cam_idx = CAMERA_NIR_IDX;
    kdrv_fb_mgr_inf_done(cam_idx, g_buf_idx[cam_idx]);
}

static void _kdp_fdfr_fb_free(void)
{
    kdrv_fb_mgr_free_inf_buf(CAMERA_RGB_IDX);
    kdrv_fb_mgr_free_inf_buf(CAMERA_NIR_IDX);
}

static int _kdp_fdfr_registeration_check(struct kdp_fdfr_comm_params *fdfr_params)
{
    if(FLAGS_KDP520_REGISTER_FDR_EVT == fdfr_params->rgst_ctx.rgst_type) {
        return _kdp_fdfr_registration(fdfr_params->rgst_ctx.user_idx);
    }
    else if((FLAGS_KDP520_AUTO_REGISTER_FDR_EVT == fdfr_params->rgst_ctx.rgst_type) ||
            (FLAGS_KDP520_AUTO_REGISTER_BOTH_FDR_EVT == fdfr_params->rgst_ctx.rgst_type)) {
        return _kdp_fdfr_auto_registeration(fdfr_params->rgst_ctx.user_idx, fdfr_params->rgst_ctx.rgst_idx, fdfr_params->rgst_ctx.num_rgst);
    }
    else if((FLAGS_KDP520_REGISTER_CFM_FDR_EVT == fdfr_params->rgst_ctx.rgst_type) ||
            (FLAGS_KDP520_REGISTER_CFM_BOTH_FDR_EVT == fdfr_params->rgst_ctx.rgst_type)){
        return _kdp_fdfr_registration_confirm(fdfr_params->rgst_ctx.user_idx, fdfr_params->rgst_ctx.num_rgst);
    }
        return -1;
}

void _kdp_chk_fdfr_draw_fps(void)
{
    static u32 ftime = 0;
    static float ffps = 0;
    if(osKernelGetTickCount() - ftime > 10000){
        float sfps = (ffps/10);
        
        if(sfps != 0){
            info_msg("draw-fps:%5.2f\n", sfps);
        }
        ffps = 0;
        ftime = osKernelGetTickCount();
    }
    else{
        ffps++;
    }
}

void _kdp_inference_chk_fdfr_fps(void)
{
    static u32 ftime = 0;
    static float ffps = 0;
    if(osKernelGetTickCount() - ftime > 10000){
        float sfps = (ffps/10);

        if(sfps != 0){
            info_msg("fdfr-fps:%5.2f\n",sfps );
        }
        ffps = 0;
        ftime = osKernelGetTickCount();
    }
    else{
        ffps++;
    }
}

fdr_result _kdp_inference_check_ack_status(fdr_result fdr_ret)
{
    u32 curr_tick;
    int _nShowimerCur = -1;
    if(FDR_RET_DB_COMP_OK == fdr_ret) {
        memcpy(&h_boundingbox, &m_boundingbox, sizeof(m_boundingbox));
        _kdp_fdfr_fb_free();
    }
    else {
        curr_tick = kdrv_current_t3_tick();
        _nShowimerCur = curr_tick - m_fdfr_tick;
        if ( _nShowimerCur >= m_face_recognition_timeout) {
            fdr_ret = FDR_RET_TIMEOUT;
            critical_msg("fdfr inference timeout =%d/%d",_nShowimerCur, m_face_recognition_timeout);
            _kdp_fdfr_fb_free();
        }
    }
    return fdr_ret;
}

static fdr_result _kdp_fdfr_inference(void)
{
    return _kdp_fdfr_single_inference();
}

static fdr_result _kdp_fdfr_inference_running(int inf_type)
{
    while(g_fdfr_run_inference == TRUE)
    {
        if(_kdp_fdfr_fb_capture() == 0)
            return FDR_RET_OOPS;

        if((g_fdr_ret = _kdp_fdfr_inference()) != FDR_RET_OOPS) {
            if(inf_type == KDP_INF_WITH_RESULT_ACK) {
                g_fdr_ret = _kdp_inference_check_ack_status(g_fdr_ret);
                if((g_fdr_ret == FDR_RET_DB_COMP_OK) || (g_fdr_ret == FDR_RET_TIMEOUT))
                    return g_fdr_ret;
            }
        }
        _kdp_frdr_fb_prepare();
        _kdp_inference_chk_fdfr_fps();
    }
    return FDR_RET_OOPS;
}

static void _kdp_fdfr_set_hint_boundingbox(void)
{
    struct video_input_params dp_params;
    kmdw_display_get_params(&dp_params);
    s_boundingbox.start_x = kdrv_display_get_frame_margin_len();
    s_boundingbox.start_y = kdrv_display_get_frame_margin_len();
    s_boundingbox.end_x = dp_params.input_xres - s_boundingbox.start_x;
    s_boundingbox.end_y = dp_params.input_yres - s_boundingbox.start_y;
    info_msg("[%s] dp_params.input_xres:%d, dp_params.input_yres:%d", __func__, dp_params.input_xres, dp_params.input_yres);
}

static void _kdp_init_fsm(kdp_fsm *pfsm)
{
    g_max_fsm_num = sizeof(g_fsm_table)/sizeof(kdp_fsm_table);
    pfsm->curr_state = KDP_FDR_FSM_IDLE;
    pfsm->pfsm_table = (kdp_fsm_table *)&g_fsm_table;
}

static int kdp_fdr_fsm_evt_handler(int evt, kdp_fdfr_comm_params *fdfr_params)
{
    kdp_fsm *pfsm = &g_fsm;
    kdp_fsm_table *pfsm_table = pfsm->pfsm_table;
    void (*evtActFunc)(int evt, kdp_fdfr_comm_params *fdfr_params) = NULL;
    int currState = pfsm->curr_state;
    int newState = 0;
    int nextState;
    int i;
    
    for(i = 0; i < g_max_fsm_num; i++)
    {
        if(evt == pfsm_table[i].evt)
        {
            newState = pfsm_table[i].state;

            /* Only approved FSM transition as below cases:
               KDP_FDR_FSM_IDLE -> KDP_FDR_FSM_INFERENCE ~ KDP_FDR_FSM_DELETE
               KDP_FDR_FSM_INFERENCE ~ KDP_FDR_FSM_DELETE -> KDP_FDR_FSM_IDLE*/
            if(currState == KDP_FDR_FSM_IDLE)
            {
                //dbg_msg(">> Enter i:%d, evt:0x%x, currState %s->%s\n", i, evt, g_kdp_fdr_fsm_str[currState], g_kdp_fdr_fsm_str[newState]);
                pfsm->curr_state = newState; //Switch current fsm
                evtActFunc = pfsm_table[i].evtActFunc;
                evtActFunc(evt, fdfr_params);
                nextState = pfsm_table[i].next_state;
                pfsm->curr_state = nextState;
                //dbg_msg(">> Leave i:%d, evt:0x%x, currState %s->%s\n", i, evt, g_kdp_fdr_fsm_str[newState], g_kdp_fdr_fsm_str[nextState]);
                break;
            }
        }
    }
    /*Wrong FSM*/
    if(i == g_max_fsm_num)
    {
        err_msg("[ERROR] Wrong FSM state %s, currState %s", g_kdp_fdr_fsm_str[newState], g_kdp_fdr_fsm_str[currState]);
        set_thread_event(fdfr_params->fdfr_ctx.caller_tid, FLAGS_FDFR_NACK);
    }
    /*Wrong Event flag*/
    if(nextState == 0)
        return -1;
    else
        return 0;
}

static void kdp_draw_thread(void *argument)
{
    uint32_t flags;
    fdr_result fdr_ret;

    while(1)
    {
        flags = osThreadFlagsWait(FLAGS_KDP520_CHANGE_DRAW_TIMEOUT_EVT, osFlagsWaitAny, g_draw_timeout_ms);
   
        if (flags == osFlagsErrorTimeout) {
            fdr_ret = kdp_fdfr_get_inference_result();
            #ifdef BOARD_96
            _kdp_draw_fdfr_result(fdr_ret);
            #endif
            //_kdp_chk_fdfr_draw_fps();
            continue;
        }
    }
}

static void kdp_inference_thread(void *argument)
{
    unsigned int flags, timeout = osWaitForever;
    kdp_all_comm_params *all_params = (kdp_all_comm_params *)argument;
    kdp_camera_comm_params *cam_params = &all_params->cam_params;
    kdp_fdfr_comm_params *fdfr_params = &all_params->fdfr_params;

    for (;;) {
        flags = osThreadFlagsWait(FLAGS_KDP520_ALL_EVENTS, osFlagsWaitAny, timeout);

        /* =========================== KDP camera ===========================  */
        if (flags & (FLAGS_KDP520_OPEN_CAMERA0_EVT | FLAGS_KDP520_OPEN_CAMERA1_EVT)) {
            cam_params->ret = _kdp_camera_open(cam_params->cam_idx);
            set_thread_event(cam_params->caller_tid, FLAGS_CAMERA_ACK);
            flags &= ~(FLAGS_KDP520_OPEN_CAMERA0_EVT | FLAGS_KDP520_OPEN_CAMERA1_EVT);
            continue;
        }

        if (flags & (FLAGS_KDP520_START_CAMERA0_EVT | FLAGS_KDP520_START_CAMERA1_EVT)) {
            cam_params->ret = _kdp_camera_start(cam_params->cam_idx);
            set_thread_event(cam_params->caller_tid, FLAGS_CAMERA_ACK);
            flags &= ~(FLAGS_KDP520_START_CAMERA0_EVT | FLAGS_KDP520_START_CAMERA1_EVT);
            continue;
        }

        if (flags & (FLAGS_KDP520_STOP_CAMERA0_EVT | FLAGS_KDP520_STOP_CAMERA1_EVT)) {
            cam_params->ret = _kdp_camera_stop(cam_params->cam_idx);
            set_thread_event(cam_params->caller_tid, FLAGS_CAMERA_ACK);
            flags &= ~(FLAGS_KDP520_STOP_CAMERA0_EVT | FLAGS_KDP520_STOP_CAMERA1_EVT);
            continue;
        }

        if (flags & (FLAGS_KDP520_CLOSE_CAMERA0_EVT | FLAGS_KDP520_CLOSE_CAMERA1_EVT)) {
            cam_params->ret = _kdp_camera_close(cam_params->cam_idx);
            set_thread_event(cam_params->caller_tid, FLAGS_CAMERA_ACK);
            flags &= ~(FLAGS_KDP520_CLOSE_CAMERA0_EVT | FLAGS_KDP520_CLOSE_CAMERA1_EVT);
            continue;
        }

        /* =========================== KDP fdfr inference =========================== */
        if(kdp_fdr_fsm_evt_handler(flags, fdfr_params) == 0) {
            continue;
        }

        if(flags) {
            err_msg("%s: unknown flags 0x%x\n", flags);
        }
    }
}

/*----------------------------------------------------------------------------------------
 *                                      Puclic FDFR API
 *----------------------------------------------------------------------------------------*/
bool kdp_fdfr_model_init(void)
{
    int j;
    int saved_count;

    if(m_fdr_inout == NULL)
        m_fdr_inout = (kapp_fid_inout_t*)kmdw_ddr_reserve(sizeof(kapp_fid_inout_t));

    if(m_fd_out == NULL)
        m_fd_out = (struct facedet_result_s*)kmdw_ddr_reserve(sizeof(struct facedet_result_s));
    if(m_lm_out == NULL)
        m_lm_out = (struct landmark_result_s*)kmdw_ddr_reserve(sizeof(struct landmark_result_s));
    if(m_fr_out == NULL)
        m_fr_out = (struct fr_result_s*)kmdw_ddr_reserve(sizeof(struct fr_result_s));

    if(m_fd_1_out == NULL)
        m_fd_1_out = (struct facedet_result_s*)kmdw_ddr_reserve(sizeof(struct facedet_result_s));
    if(m_lm_1_out == NULL)
        m_lm_1_out = (struct landmark_result_s*)kmdw_ddr_reserve(sizeof(struct landmark_result_s));
    if(m_fr_1_out == NULL)
        m_fr_1_out = (struct fr_result_s*)kmdw_ddr_reserve(sizeof(struct fr_result_s));
    

    if(m_db_data == NULL)
        m_db_data = (kapp_db_user_data_t*)kmdw_ddr_reserve(sizeof(kapp_db_user_data_t));
    //todo
    if (db_tmp_fr == NULL)
        db_tmp_fr = (float *)kmdw_ddr_reserve(sizeof(uint8_t)*REG_TEMP_FR_SIZE);
    if (db_tmp_lm == NULL)
        db_tmp_lm = (uint32_t *)kmdw_ddr_reserve(sizeof(uint8_t)*REG_TEMP_LM_SIZE);

    if (db_tmp_1_fr == NULL)
        db_tmp_1_fr = (float *)kmdw_ddr_reserve(sizeof(uint8_t)*REG_TEMP_FR_SIZE);
    if (db_tmp_1_lm == NULL)
        db_tmp_1_lm = (uint32_t *)kmdw_ddr_reserve(sizeof(uint8_t)*REG_TEMP_LM_SIZE);

    saved_count = kapp_fid_init();
    kapp_config_image(KAPP_IMG_SRC_RGB, RGB_IMG_SOURCE_W, RGB_IMG_SOURCE_H, 3,
                         KAPP_IMAGE_FORMAT_RGB565, 0/*default addr*/, 0);
    kapp_config_image(KAPP_IMG_SRC_NIR, NIR_IMG_SOURCE_CROP_H, NIR_IMG_SOURCE_CROP_W, 1,
                         KAPP_IMAGE_FORMAT_RAW8, 0/*default addr*/, 1);

    kapp_db_init(KAPP_DB_FLASH_ADDR);     //load db from flash

    //init m_sample_db.add_tag according to existed data in DB
    for (j=0 ; j<MAX_USER;j++) {
        if(saved_count == 0)
            break;

        if(kapp_db_slot_is_used(j)) {
            saved_count--;
        }
    }
    return TRUE;
}

fdr_result kdp_fdfr_get_inference_result(void)
{
    return g_fdr_ret;
}

void kdp_fdfr_set_draw_fps(int fps)
{
    if(fps >= 5 && fps <= 30)
        g_draw_timeout_ms = (uint32_t)(1000/fps);
}

int kdp_fdfr_draw_thread_create(void)
{
    int ret = -1;
    if (tid_draw != NULL)
        return ret;

    if((tid_draw = osThreadNew(kdp_draw_thread, NULL, NULL)) != NULL) {
        osThreadSetPriority(tid_draw, osPriorityNormal);
        ret = 0;
    }
    return ret;
}

int kdp_fdfr_inference_thread_create(void)
{
    int ret = -1;
    if (NULL != tid_inference)
        return ret;

    if(NULL != (tid_inference = osThreadNew(kdp_inference_thread, (void*)&m_kdp_all_comm_params, NULL))) {
        osThreadSetPriority (tid_inference, osPriorityNormal);
        power_manager_register(PM_DEVICE_NCPU_INFERENCE, &pm_ncpu_inference);
        _kdp_init_fsm(&g_fsm);
        ret = 0;
    }
    return ret;
}

void kdp_fdfr_set_boundingbox_mode(int mode)
{
    g_draw_boundingbox_mode = mode;
}

void kdp_fdfr_inference_start(void)
{
    memset(&m_boundingbox, 0, sizeof(m_boundingbox));
    memset(&h_boundingbox, 0, sizeof(h_boundingbox));
    _kdp_fdfr_set_hint_boundingbox();
    g_fdfr_run_inference = TRUE;
    g_draw_timeout_ms = 50; //20 fps
    set_thread_event(tid_draw, FLAGS_KDP520_CHANGE_DRAW_TIMEOUT_EVT);
}

void kdp_fdfr_inference_stop(void)
{
    memset(&s_boundingbox, 0, sizeof(s_boundingbox));
    g_fdfr_run_inference = FALSE;
    g_draw_timeout_ms = osWaitForever;   
    set_thread_event(tid_draw, FLAGS_KDP520_CHANGE_DRAW_TIMEOUT_EVT);
}

bool kdp_fdfr_inference_is_running(void)
{
    return g_fdfr_run_inference;
}

int kdp_fdfr_get_available_db_idx(void)
{
    return kapp_db_get_available_ID_slot();
}

int kdp_fdfr_get_remaining_fmap_cnt(int user_idx)
{
    return (MAX_FID - kapp_db_get_fm0_cnt(user_idx));
}

void kdp_fdfr_recognition_variable_reset(void)
{
    m_iCurExpoTime = Max_EXPO_TIME;
    kmdw_camera_set_exp_time(CAMERA_NIR_IDX, (m_iCurExpoTime&0x0000FF00)>>8, m_iCurExpoTime&0xFF);
    //led_flag = 0;
    //RgbLedFlag = 0;
    kmdw_camera_led_switch(CAMERA_NIR_IDX, 0);
    //rgb_led_close();
    //distance_count = 1;
    kmdw_camera_set_gain(CAMERA_NIR_IDX, 0x27, 0x24);
    ////dbg_msg("close nir led \n");
    //initTile_flag = 0;
    //preprocess_cnt = 0;
    //recognition_cnt = 0;
    nir_frm_cnt = 0;
    //rgb_light_cnt = 0;
    //nir_mode = 10;
//    TgtLedPwm = 0;
//    CurLedPwm = 0;
}

void kdp_fdr_do_inference(int evt, kdp_fdfr_comm_params *fdfr_params)
{
    if(evt == FLAGS_KDP520_START_INFERENCE_LOOP_EVT) {
        _kdp_fdfr_inference_running(KDP_INF_WITHOUT_RESULT_ACK);
        osThreadFlagsClear(evt);
    }
    else if(evt == FLAGS_KDP520_START_INFERENCE_EVT) {
        fdfr_params->fdfr_ctx.ret = _kdp_fdfr_inference_running(KDP_INF_WITH_RESULT_ACK);
        set_thread_event(fdfr_params->fdfr_ctx.caller_tid, FLAGS_FDFR_ACK);
        osThreadFlagsClear(evt);
    }
    else if(evt == FLAGS_KDP520_STOP_INFERENCE_LOOP_EVT) {
        _kdp_fdfr_fb_free();
        set_thread_event(fdfr_params->fdfr_ctx.caller_tid, FLAGS_FDFR_ACK);
        osThreadFlagsClear(evt);
    }
}

void kdp_fdr_do_register(int evt, kdp_fdfr_comm_params *fdfr_params)
{
    if((evt == FLAGS_KDP520_REGISTER_FDR_EVT) ||
       (evt == FLAGS_KDP520_AUTO_REGISTER_FDR_EVT) ||
       (evt == FLAGS_KDP520_REGISTER_CFM_FDR_EVT) ||
       (evt == FLAGS_KDP520_AUTO_REGISTER_BOTH_FDR_EVT) ||
       (evt == FLAGS_KDP520_REGISTER_CFM_BOTH_FDR_EVT)) {
        _kdp_fdfr_fb_capture();
        fdfr_params->rgst_ctx.ret = _kdp_fdfr_registeration_check(fdfr_params);
        _kdp_fdfr_fb_free();
        set_thread_event(fdfr_params->rgst_ctx.caller_tid, FLAGS_FDFR_ACK);
        osThreadFlagsClear(evt);
    }
}

void kdp_fdr_do_cancel_register(int evt, kdp_fdfr_comm_params *fdfr_params)
{
    if(evt == FLAGS_KDP520_CANCEL_REGISTER_FDR_EVT) {
        fdfr_params->rgst_cancel_ctx.ret = _kdp_fdfr_cancel_registeration();
        set_thread_event(fdfr_params->rgst_cancel_ctx.caller_tid, FLAGS_FDFR_ACK);
        osThreadFlagsClear(evt);
    }
}

void kdp_fdr_do_add(int evt, kdp_fdfr_comm_params *fdfr_params)
{
    if(evt == FLAGS_KDP520_ADD_FDR_FACE_EVT) {
        fdfr_params->add_ctx.ret = _kdp_fdfr_add_user(fdfr_params->add_ctx.user_idx);
        set_thread_event(fdfr_params->add_ctx.caller_tid, FLAGS_FDFR_ACK);
        osThreadFlagsClear(evt);
    }
}

void kdp_fdr_do_delete(int evt, kdp_fdfr_comm_params *fdfr_params)
{
    if(evt == FLAGS_KDP520_DEL_FDR_FACE_EVT) {
        fdfr_params->del_ctx.ret = _kdp_fdfr_del_user(fdfr_params->del_ctx.all, fdfr_params->del_ctx.user_idx);
        set_thread_event(fdfr_params->del_ctx.caller_tid, FLAGS_FDFR_ACK);
        osThreadFlagsClear(evt);
    }
}


/*----------------------------------------------------------------------------------------
 *                                  Puclic camera API
 *----------------------------------------------------------------------------------------*/
int kdp_camera_setting(uint8_t cam_idx, unsigned int width, unsigned int height, unsigned int pixelformat)
{
    if(TRUE == m_kdp_cam_ctx[cam_idx].init) {
        //dbg_msg("[%s] camera index %d is already initialized", __func__, cam_idx);
        return -1;
    }
    m_kdp_cam_ctx[cam_idx].init = TRUE;
    m_kdp_cam_ctx[cam_idx].width = width;
    m_kdp_cam_ctx[cam_idx].height = height;
    m_kdp_cam_ctx[cam_idx].pixelformat = pixelformat;
    return 0;
}

int kdp_get_camera_ctx_state(uint8_t cam_idx)
{
    return m_kdp_cam_ctx[cam_idx].state;
}

osThreadId_t kdp_get_inference_thread_id(void)
{
    return tid_inference;
}

void kdp_camera_enable_snapshot_dp(void)
{
    kdrv_display_set_snapshot_preview_en(TRUE);
    g_en_snapshot_fdfr[0] = FALSE;
    g_en_snapshot_fdfr[1] = FALSE;
}

int kdp_camera_snapshot_save(void)
{
    g_snapshot_idx = ((g_snapshot_idx + 1) % KDP_FLASH_SNAPSHOT_NUMS);
    kdp_memxfer_module.ddr_to_flash((uint32_t)g_snapshot_idx_addr[g_snapshot_idx], (uint32_t)KDP_DDR_SNAPSHOT_RGB_IMG_ADDR, KDP_DDR_SNAPSHOT_RGB_IMG_SIZE);
    info_msg("[Snapshot]Save to FLASH index:%d addr:0x%p\n", g_snapshot_idx, g_snapshot_idx_addr[g_snapshot_idx]);
    return g_snapshot_idx;
}

bool kdp_camera_review_snapshot_history(int idx)
{
    if(idx >= KDP_FLASH_SNAPSHOT_NUMS) {
        err_msg("[%s][ERR] snapshot index is over than %d\n", __func__, KDP_FLASH_SNAPSHOT_NUMS);
        return FALSE;
    }
    kdp_memxfer_module.flash_to_ddr((uint32_t)KDP_DDR_SNAPSHOT_RGB_IMG_ADDR, (uint32_t)g_snapshot_idx_addr[idx], KDP_DDR_SNAPSHOT_RGB_IMG_SIZE);
    kdrv_display_set_review_snapshot_en(TRUE);
    return TRUE;
}

void kdp_camera_enable_snapshot_review(void)
{
    kdrv_display_set_review_snapshot_en(TRUE);
}

void kdp_camera_disable_snapshot_review(void)
{
    kdrv_display_set_review_snapshot_en(FALSE);
}

void kdp_camera_enable_snapshot_fdfr(void)
{
    g_en_snapshot_fdfr[0] = TRUE;
    g_en_snapshot_fdfr[1] = TRUE;
    kdrv_display_set_snapshot_preview_en(FALSE);
}

void kdp_fdfr_snapshot_save(void)
{
    kdp_memxfer_module.ddr_to_flash((uint32_t)KDP_FLASH_SNAPSHOT_IMG_RGB_FDFR_ADDR, (uint32_t)KDP_DDR_SNAPSHOT_RGB_IMG_ADDR, KDP_FLASH_RGB_SIZE);
    info_msg("[Snapshot]rgb image for fdfr save flash...ok\n");
    kdp_memxfer_module.ddr_to_flash((uint32_t)KDP_FLASH_SNAPSHOT_IMG_NIR_FDFR_ADDR, (uint32_t)KDP_DDR_SNAPSHOT_NIR_IMG_ADDR, KDP_FLASH_NIR_SIZE);
    info_msg("[Snapshot]nir image for fdfr save flash...ok\n");
}

bool kdp_fdfr_snapshot(uint8_t cam_idx, uint32_t addr)
{
    uint32_t dp_ui_fb = 0;
    if(cam_idx >= IMGSRC_NUM)
        return FALSE;

    if(g_en_snapshot_fdfr[cam_idx] == FALSE)
        return FALSE;

    if(cam_idx == CAMERA_RGB_IDX){
        memcpy((void *)KDP_DDR_SNAPSHOT_RGB_IMG_ADDR, (void *)addr, KDP_DDR_SNAPSHOT_RGB_IMG_SIZE);

        /* dp_ui_fb is used to show the snapshot fdfr frame buffer with boudingbox if kdrv_display_get_review_snapshot_en() == TRUE */
        dp_ui_fb = kmdw_video_renderer_get_buffer_addr();
        memcpy((void *)dp_ui_fb, (void *)KDP_DDR_SNAPSHOT_RGB_IMG_ADDR, KDP_DDR_SNAPSHOT_RGB_IMG_SIZE);
        info_msg("Snapshot fdfr rgb image done...\n");
    }
    else if(cam_idx == CAMERA_NIR_IDX){
        memcpy((void *)KDP_DDR_SNAPSHOT_NIR_IMG_ADDR, (void *)addr, KDP_DDR_SNAPSHOT_NIR_IMG_SIZE);
        info_msg("Snapshot fdfr nir image done...\n");
    }
    g_en_snapshot_fdfr[cam_idx] = FALSE;
    return TRUE;
}

void kdp_fdfr_review_snapshot(void)
{
    kdp_memxfer_module.flash_to_ddr((uint32_t)KDP_DDR_SNAPSHOT_RGB_IMG_ADDR, (uint32_t)KDP_FLASH_SNAPSHOT_IMG_RGB_FDFR_ADDR, KDP_FLASH_RGB_SIZE);
    kdp_memxfer_module.flash_to_ddr((uint32_t)KDP_DDR_SNAPSHOT_NIR_IMG_ADDR, (uint32_t)KDP_FLASH_SNAPSHOT_IMG_NIR_FDFR_ADDR, KDP_FLASH_NIR_SIZE);
    kdrv_display_set_review_snapshot_en(TRUE);
}

uint32_t kdp_fdfr_get_snapshot_addr(uint8_t cam_idx)
{
    if((cam_idx == CAMERA_RGB_IDX) && (g_en_snapshot_fdfr[cam_idx] == FALSE)) {
        return KDP_DDR_SNAPSHOT_RGB_IMG_ADDR;
    }
    else if((cam_idx == CAMERA_NIR_IDX) && (g_en_snapshot_fdfr[cam_idx] == FALSE)) {
        return KDP_DDR_SNAPSHOT_NIR_IMG_ADDR;
    }
    return 0;   //buffer not ready
}
