'''
 @file bin_gen.py
 @brief Generate flash final bin file from other seperate bin files.
'''

import argparse
import io
import os
import code
import sys
import pdb
import numpy as np

BIN_FROM_PROJECT=1
SPL_CPU_ONLY=0
MAX_MODEL_SIZE=0x1000000

flash_bin_info = [
                  [0x00000000, 8*1024,  './flash_bin/boot_spl.bin'],
                  [0x00002000, 88*1024, './flash_bin/fw_scpu.bin'],
                  [0x00018000, 64*1024, './flash_bin/fw_ncpu.bin'],
                  [0x00300000, 232,     './flash_bin/fw_info.bin'],
                  [0x00301000, 9850100, './flash_bin/all_models.bin'],
                  ]

def file_read_binary(src):
     ''' Load depth file '''
     with open(src, "rb") as f:
         data = f.read()
         return data

def write_data_to_binary_file(ofile, data):
    with io.open(ofile, 'wb') as fo:
        fo.write(bytearray(data))

def bin_gen(bin_info, ofile):
    bin_len = len(bin_info)
    model_size = 0;
    if os.path.isfile(bin_info[-1][2]):
        model_size = os.path.getsize(bin_info[-1][2])
        #print('model size = %d' %model_size)
        if(model_size >= MAX_MODEL_SIZE):
            print('[ERROR] model file size is too big')
            sys.exit(-1)
    else:
        print('model file is not available')
        sys.exit(-1)

    total_bin_size = bin_info[-1][0] + model_size
    bin_data = np.zeros(total_bin_size, dtype=np.uint8)
    bin_data.fill(0xff)
    for bin_file in bin_info:
        print('start parsing, start: 0x{:08X}, size: {:08X}, file={:s}'.format(bin_file[0], bin_file[1], bin_file[2]))
        bin_start_addr = bin_file[0]
        bin_file = bin_file[2]

        if os.path.isfile(bin_file) == 0:
            print('{:s} is not exists'.format(bin_file))
        else:
            rdata = file_read_binary(bin_file)
            rdata = list(rdata)
            rdata = np.array(rdata)
            bin_data[bin_start_addr:bin_start_addr + len(rdata)] = rdata

    write_data_to_binary_file(ofile, bin_data)

if __name__ == '__main__':
    ### please make sure image directory and model path area correct
    print('---------------------------------------------------------------')
    print('Generate final flash bin file')
    print('Bin file information list')
    print("== Generate flash image binary file ==")
    if SPL_CPU_ONLY == 1:
        flash_bin_info = flash_bin_info[:3]
    bin_gen(flash_bin_info, 'flash_image.bin')

